<?php

namespace xenMade\LD\Admin\Controller;

use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

class Category extends AbstractController
{
	public function actionIndex()
	{
		return $this->getCategoryTreePlugin()->actionList(['permissionContentType' => 'ld_category']);
	}

	public function actionEdit(ParameterBag $params)
	{
        /** @var \xenMade\LD\Entity\Category $category */
		$category = $this->assertCategoryExists($params->category_id);
		return $this->categoryAddEdit($category);
	}

	public function actionAdd()
	{
	    /** @var \xenMade\LD\Entity\Category $category */
		$category = $this->em()->create('xenMade\LD:Category');
		$category->parent_category_id = $this->filter('parent_category_id', 'uint');
		
		return $this->categoryAddEdit($category);
	}

    public function categoryAddEdit(\xenMade\LD\Entity\Category $category)
    {
        $categoryRepo = $this->getCategoryRepo();
        $categories = $categoryRepo->findCategoryList()->fetch();
        $categoryTree = $categoryRepo->createCategoryTree($categories);


        /** @var \xenMade\LD\Repository\LinkPrefix $prefixRepo */
        $prefixRepo = $this->repository('xenMade\LD:LinkPrefix');
        $availablePrefixes = $prefixRepo->findPrefixesForList()->fetch();
        $availablePrefixes = $availablePrefixes->pluckNamed('title', 'prefix_id');

        /** @var \xenMade\LD\Repository\LinkField $fieldRepo */
        $fieldRepo = $this->repository('xenMade\LD:LinkField');
        $availableFields = $fieldRepo->findFieldsForList()->fetch();
        $availableFields = $availableFields->pluckNamed('title', 'field_id');


        if ($category->thread_prefix_id && $category->ThreadForum)
        {
            $threadPrefixes = $category->ThreadForum->getPrefixesGrouped();
        }
        else
        {
            $threadPrefixes = [];
        }


        $viewParams = [
            'category' => $category,
            'categoryTree' => $categoryTree,
            'availableFields' => $availableFields,
            'availablePrefixes' => $availablePrefixes,

            'forumOptions' => $this->repository('XF:Node')->getNodeOptionsData(false, 'Forum'),
            'threadPrefixes' => $threadPrefixes,
        ];
        return $this->view('xenMade\LD:Category\Edit', 'ld_category_edit', $viewParams);
    }

    public function actionSave(ParameterBag $params)
	{
		$this->assertPostOnly();

        /** @var \xenMade\LD\Entity\Category $category */
		if ($params->category_id)
		{
			$category = $this->assertCategoryExists($params->category_id);
		}
		else
		{
			$category = $this->em()->create('xenMade\LD:Category');
		}

		$this->categorySaveProcess($category)->run();

		return $this->redirect(
			$this->buildLink('link-directory/categories') . $this->buildLinkHash($category->getEntityId())
		);
	}

	public function actionDelete(ParameterBag $params)
	{
		return $this->getCategoryTreePlugin()->actionDelete($params);
	}

	public function actionSort()
	{
		return $this->getCategoryTreePlugin()->actionSort();
	}

	public function actionPermissions(ParameterBag $params)
	{
		return $this->getCategoryPermissionPlugin()->actionList($params);
	}

	public function actionPermissionsEdit(ParameterBag $params)
	{
		return $this->getCategoryPermissionPlugin()->actionEdit($params);
	}

	public function actionPermissionsSave(ParameterBag $params)
	{
		return $this->getCategoryPermissionPlugin()->actionSave($params);
	}



    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertAdminPermission('ldManager');
    }

    /**
     * @return \xenMade\LD\ControllerPlugin\CategoryTree
     */
    protected function getCategoryTreePlugin()
    {
        return $this->plugin('xenMade\LD:CategoryTree');
    }

    protected function categorySaveProcess(\XF\Entity\AbstractCategoryTree $category)
    {
        $entityInput = $this->filter([
            'title' => 'str',
            'description' => 'str',
            'parent_category_id' => 'uint',
            'display_order' => 'uint',
            'allow_link' => 'bool',
            'always_moderate_create' => 'bool',
            'always_moderate_update' => 'bool',
            'backlink' => 'str',
            'backlink_location' => 'bool',
            'min_tags' => 'uint',
            'require_prefix' => 'bool',
            'thread_node_id' => 'uint',
            'thread_prefix_id' => 'uint',
        ]);

        $form = $this->formAction();
        $form->basicEntitySave($category, $entityInput);

        $prefixIds = $this->filter('available_prefixes', 'array-uint');
        $form->complete(function() use ($category, $prefixIds)
        {
            /** @var \xenMade\LD\Repository\CategoryPrefix $repo */
            $repo = $this->repository('xenMade\LD:CategoryPrefix');
            $repo->updateContentAssociations($category->category_id, $prefixIds);
        });

        $fieldIds = $this->filter('available_fields', 'array-str');
        $form->complete(function() use ($category, $fieldIds)
        {
            /** @var \xenMade\LD\Repository\CategoryField $repo */
            $repo = $this->repository('xenMade\LD:CategoryField');
            $repo->updateContentAssociations($category->category_id, $fieldIds);
        });

        return $form;
    }

    protected function assertCategoryExists($id, $with = null, $phraseKey = null)
	{
		return $this->assertRecordExists('xenMade\LD:Category', $id, $with, $phraseKey);
	}

    /**
     * @return \xenMade\LD\ControllerPlugin\CategoryPermission
     */
    protected function getCategoryPermissionPlugin()
    {
        /** @var \xenMade\LD\ControllerPlugin\CategoryPermission $plugin */
        $plugin = $this->plugin('xenMade\LD:CategoryPermission');
        $plugin->setFormatters('xenMade\LD:Category\Permission%s', 'ld_category_permission_%s');
        $plugin->setRoutePrefix('link-directory/categories/permissions');

        return $plugin;
    }

    /**
     * @return \xenMade\LD\Repository\Category
     */
	protected function getCategoryRepo()
	{
		return $this->repository('xenMade\LD:Category');
	}
}