<?php

namespace xenMade\LD\Admin\Controller;

use XF\Admin\Controller\AbstractField;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;

class Field extends AbstractField
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('ldManager');
	}

	protected function getClassIdentifier()
	{
		return 'xenMade\LD:LinkField';
	}

	protected function getLinkPrefix()
	{
		return 'link-directory/fields';
	}

	protected function getTemplatePrefix()
	{
		return 'ld_link_field';
	}

	protected function fieldAddEditResponse(\XF\Entity\AbstractField $field)
	{
		$reply = parent::fieldAddEditResponse($field);

		if ($reply instanceof \XF\Mvc\Reply\View)
		{
			/** @var \xenMade\LD\Repository\Category $categoryRepo */
			$categoryRepo = $this->repository('xenMade\LD:Category');

			$categories = $categoryRepo->findCategoryList()->fetch();
			$categoryTree = $categoryRepo->createCategoryTree($categories);

			/** @var \XF\Mvc\Entity\ArrayCollection $fieldAssociations */
			$fieldAssociations = $field->getRelationOrDefault('CategoryFields', false);

			$reply->setParams([
				'categoryTree' => $categoryTree,
				'categoryIds' => $fieldAssociations->pluckNamed('category_id')
			]);
		}

		return $reply;
	}

	protected function saveAdditionalData(FormAction $form, \XF\Entity\AbstractField $field)
	{
		$categoryIds = $this->filter('category_ids', 'array-uint');

		/** @var \xenMade\LD\Entity\LinkField $field */
		$form->complete(function() use ($field, $categoryIds)
		{
			/** @var \xenMade\LD\Repository\CategoryField $repo */
			$repo = $this->repository('xenMade\LD:CategoryField');
			$repo->updateFieldAssociations($field, $categoryIds);
		});

		return $form;
	}
}