<?php

namespace xenMade\LD\Admin\Controller;

use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

class InsertUrl extends AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('ldManager');
	}

    public function actionIndex()
    {
        if ($this->isPost())
        {
            $categoryId = $this->filter('category_id', 'uint');
            if(!$categoryId)
            {
                throw $this->exception($this->error(\XF::phrase('requested_category_not_found')));
            }

            $urls = $this->filter('urls', 'str');
            $urls = preg_split('#(\r\n|\n|\r|\s+|,)#s', trim($urls), -1, PREG_SPLIT_NO_EMPTY);


            $fail_urls = [];
            foreach ($urls as $key => $url)
            {
                /** @var \XF\Validator\Url $urlValidator */
                $urlValidator = \XF::app()->validator('Url');

                if (!$urlValidator->isValid($url))
                {
                    $fail_urls[] = $url;
                    unset($urls[$key]);
                }
            }

            reset($urls);

            $this->app->jobManager()->enqueueUnique('InsertUrls', 'xenMade\LD:InsertUrl', [
                'urls' => $urls,
                'count' => count($urls),
                'category_id' => $categoryId,
                'fail_urls' => $fail_urls,
            ]);

            return $this->redirect($this->buildLink('link-directory/add-mass-urls', ['urls' => \XF::app()->offsetExists('testXF')], ['success' => true]));
        }

        /** @var \xenMade\LD\Repository\Category $categoryRepo */
        $categoryRepo = $this->repository('xenMade\LD:Category');

        $categories = $categoryRepo->findCategoryList()->fetch();
        $categoryTree = $categoryRepo->createCategoryTree($categories);


        $failUrls = \XF::app()->simpleCache()->LD->FailUrls;

        if($failUrls)
        {
            \XF::app()->simpleCache()->LD->FailUrls = [];
        }

        if(is_array($failUrls))
        {
            $failUrls = implode("\n", $failUrls);
        }

        $viewParams = [
            'success' => $this->filter('success', 'bool'),
            'categoryTree' => $categoryTree,
            'urls' => $failUrls,
        ];

        return $this->view('xenMade\LD:Link\Urls', 'ld_mass_insert_url', $viewParams);
    }
}