<?php

namespace xenMade\LD\Admin\Controller;

use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

class Upgrade extends AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('ldManager');
	}

    public function actionIndex()
    {
        $upgrades = $this->getUpgradeRepo()->findUpgradeForList();

        $viewParams = [
            'upgrades' => $upgrades->fetch(),
            'totals' => $upgrades->total(),
        ];

        return $this->view('xenMade\LD:Upgrade\View', 'ld_link_upgrade_list', $viewParams);
    }

    public function actionAdd()
    {
        $upgrade = $this->em()->create('xenMade\LD:LinkUpgrade');

        return $this->upgradeAddEdit($upgrade);
    }

    public function actionEdit(ParameterBag $params)
    {
        $paymentRepo = $this->repository('XF:Payment');
        if (!$paymentRepo->findPaymentProfilesForList()->total())
        {
            throw $this->exception($this->error(\XF::phrase('please_create_at_least_one_payment_profile_before_continuing')));
        }

        $upgrade = $this->assertUpgradeExists($params->upgrade_id);

        return $this->upgradeAddEdit($upgrade);
    }

    public function actionSave(ParameterBag $params)
    {
        $this->assertPostOnly();

        if ($params->upgrade_id)
        {
            $upgrade = $this->assertUpgradeExists($params->upgrade_id);
        }
        else
        {
            $upgrade = $this->em()->create('xenMade\LD:LinkUpgrade');
        }

        $form = $this->upgradeSaveProcess($upgrade);
        $form->run();

        return $this->redirect($this->buildLink('link-directory/upgrades'));
    }

    public function actionDelete(ParameterBag $params)
    {
        $upgrade = $this->assertUpgradeExists($params->upgrade_id);

        if ($this->isPost())
        {
            $upgrade->delete();

            return $this->redirect($this->buildLink('link-directory/upgrades'));
        }
        else
        {
            $viewParams = [
                'upgrade' => $upgrade
            ];

            return $this->view('xenMade\LD:Upgrade\Delete', 'ld_link_upgrade_delete', $viewParams);
        }
    }

    public function actionActiveUpgrades(ParameterBag $params)
    {
        $upgradeRepo = $this->getUpgradeRepo();

        $activeUpgrades = $upgradeRepo->findActiveLinkUpgradesForList()
            ->with('Upgrade', true);

        $linkParams = [];
        $filter = $this->prepareActiveExpiredList($activeUpgrades, $params, $linkParams);

        $page = $this->filterPage();
        $perPage = 20;

        $activeUpgrades->limitByPage($page, $perPage);
        $totalActive = $activeUpgrades->total();

        $viewParams = [
            'activeUpgrades' => $activeUpgrades->fetch(),

            'page' => $page,
            'perPage' => $perPage,
            'filter' => $filter,
            'linkParams' => $linkParams,
            'totalActive' => $totalActive,
            'typeId' => 'active'
        ];

        return $this->view('xenMade\LD:Link\ActiveUpgrades', 'ld_link_upgrade_active_list', $viewParams);
    }

    public function actionExpiredUpgrades(ParameterBag $params)
    {
        $upgradeRepo = $this->getUpgradeRepo();

        $expiredUpgrades = $upgradeRepo->findExpiredLinkUpgradesForList()
            ->with('Upgrade', true);

        $linkParams = [];
        $filter = $this->prepareActiveExpiredList($expiredUpgrades, $params, $linkParams);

        $page = $this->filterPage();
        $perPage = 20;

        $expiredUpgrades->limitByPage($page, $perPage);
        $totalExpired = $expiredUpgrades->total();

        $viewParams = [
            'expiredUpgrades' => $expiredUpgrades->fetch(),

            'page' => $page,
            'perPage' => $perPage,
            'filter' => $filter,
            'linkParams' => $linkParams,
            'totalExpired' => $totalExpired,
            'typeId' => 'expired'
        ];

        return $this->view('xenMade\LD:Link\ExpiredUpgrades', 'ld_link_upgrade_expired_list', $viewParams);
    }

    public function actionActiveUpgradesedit()
    {
        $activeUpgrade = $this->assertRecordExists('xenMade\LD:LinkUpgradeActive', $this->filter('upgrade_record_id', 'uint'), ['Upgrade', 'Link']);

        if ($this->isPost())
        {
            $upgradeService = $this->service('xenMade\LD:Link\Upgrade', $activeUpgrade->Upgrade, $activeUpgrade->Link);
            $upgradeService->ignoreUnpurchasable(true);
            $endType = $this->filter('end_type', 'str');

            if ($endType == 'permanent')
            {
                $upgradeService->setEndDate(0);
            }
            else
            {
                $upgradeService->setEndDate($this->filter('end_date', 'datetime'));
            }

            $upgradeService->upgrade();

            return $this->redirect($this->buildLink('link-directory/active-upgrades'));
        }
        else
        {
            $viewParams = [
                'activeUpgrade' => $activeUpgrade
            ];

            return $this->view('xenMade\LD:Link\EditActive', 'ld_link_upgrade_active_edit', $viewParams);
        }
    }

    public function actionActiveUpgradesdowngrade()
    {
        $activeUpgrade = $this->assertRecordExists('xenMade\LD:LinkUpgradeActive', $this->filter('upgrade_record_id', 'uint'), ['Upgrade', 'Link']);

        if ($this->isPost())
        {
            $downgradeService = $this->service('xenMade\LD:Link\Downgrade', $activeUpgrade->Upgrade, $activeUpgrade->Link);
            $downgradeService->setSendAlert(false);
            $downgradeService->downgrade();

            return $this->redirect($this->buildLink('link-directory/active-upgrades'));
        }
        else
        {
            $viewParams = [
                'activeUpgrade' => $activeUpgrade
            ];
            return $this->view('xenMade\LD:Link\Downgrade', 'ld_link_upgrade_active_downgrade', $viewParams);
        }
    }

    protected function prepareActiveExpiredList(\XF\Mvc\Entity\Finder $finder, ParameterBag $params, array &$linkParams)
    {
        $userUpgrade = null;
        if ($params->upgrade_id)
        {
            $userUpgrade = $this->assertUpgradeExists($params->upgrade_id);
            $finder->where('user_upgrade_id', $params->upgrade_id);
        }

        $order = $this->filter('order', 'str');
        if ($order)
        {
            if ($order == 'username')
            {
                $finder->order('User.username', 'ASC');
                $linkParams['order'] = $order;
                $linkParams['direction'] = '';
            }
            else
            {
                $direction = $this->filter('direction', 'str');
                $finder->order($order, $direction);
                $linkParams['order'] = $order;
                $linkParams['direction'] = $direction;
            }
        }

        if ($username = $this->filter('username', 'str'))
        {
            $user = $this->em()->findOne('XF:User', ['username' => $username]);
            if (!$user)
            {
                throw $this->exception($this->error(\XF::phrase('requested_user_not_found')));
            }
            $finder->where('user_id', $user->user_id);
            $linkParams['username'] = $username;
        }

        return $userUpgrade;
    }

    protected function upgradeSaveProcess(\xenMade\LD\Entity\LinkUpgrade $upgrade)
    {
        $form = $this->formAction();

        $input = $this->filter([
            'title' => 'str',
            'description' => 'str',
            'display_order' => 'uint',
            'recurring' => 'bool',
            'cost_amount' => 'unum',
            'cost_currency' => 'str',
            'length_amount' => 'uint',
            'length_unit' => 'string',
            'payment_profile_ids' => 'array-uint',
            'can_purchase' => 'bool'
        ]);
        $form->basicEntitySave($upgrade, $input);

        $form->setup(function() use($upgrade)
        {
            if ($this->filter('length_type', 'str') == 'permanent')
            {
                $upgrade->length_amount = 0;
                $upgrade->length_unit = '';
            }
        });

        $categoryIds = $this->filter('category_ids', 'array-uint');

        /** @var \xenMade\LD\Entity\LinkUpgrade $upgrade */
        $form->complete(function() use ($upgrade, $categoryIds)
        {
            /** @var \xenMade\LD\Repository\CategoryUpgrade $repo */
            $repo = $this->repository('xenMade\LD:CategoryUpgrade');
            $repo->updateUpgradeAssociations($upgrade, $categoryIds);
        });

        return $form;
    }

    protected function upgradeAddEdit(\xenMade\LD\Entity\LinkUpgrade $upgrade)
    {
        $currencyData = \XF::app()->data('XF:Currency')->getCurrencyData();

        $paymentRepo = $this->repository('XF:Payment');
        $paymentProfiles = $paymentRepo->findPaymentProfilesForList()->fetch();


        /** @var \xenMade\LD\Repository\Category $categoryRepo */
        $categoryRepo = $this->repository('xenMade\LD:Category');

        $categories = $categoryRepo->findCategoryList()->fetch();
        $categoryTree = $categoryRepo->createCategoryTree($categories);

        /** @var \XF\Mvc\Entity\ArrayCollection $fieldAssociations */
        $fieldAssociations = $upgrade->getRelationOrDefault('CategoryUpgrades', false);

        $viewParams = [
            'upgrade' => $upgrade,
            'currency_data' => $currencyData,
            'profiles' => $paymentProfiles,
            'totalProfiles' => $paymentProfiles->count(),

            'categoryTree' => $categoryTree,
            'categoryIds' => $fieldAssociations->pluckNamed('category_id')
        ];

        return $this->view('xenMade\LD:Upgrade\Edit', 'ld_link_upgrade_edit', $viewParams);
    }

    /**
     * @return \xenMade\LD\Entity\LinkUpgrade
     */
    protected function assertUpgradeExists($id, $with = null, $phraseKey = null)
    {
        return $this->assertRecordExists('xenMade\LD:LinkUpgrade', $id, $with, $phraseKey);
    }

    /**
     * @return \xenMade\LD\Repository\LinkUpgrade
     */
    protected function getUpgradeRepo()
    {
        return $this->repository('xenMade\LD:LinkUpgrade');
    }
}