<?php

namespace xenMade\LD\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;

class PageData extends AbstractPlugin
{
    protected $charset;

    public function fetchData($url)
    {
        $tags = null;
        $title = null;
        $description = null;

        try
        {
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
            curl_setopt($ch, CURLOPT_TIMEOUT, 10);

            curl_setopt($ch, CURLOPT_VERBOSE, false);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_AUTOREFERER, true);
            curl_setopt($ch, CURLOPT_REFERER, $url);
            curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
            curl_setopt($ch, CURLOPT_HEADER, true);
            curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:42.0) Gecko/20100101 Firefox/42.0');
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($ch, CURLOPT_FRESH_CONNECT, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($ch, CURLOPT_NOBODY, false);

            $html = curl_exec($ch);
            $statusCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

            $header = $this->getcUrlHeaders($html);
            $this->charset = $this->getCharset($header, $html);
            $title = $this->getTitle($html);
            $description = $this->getDescription($html);
        }
        catch (\Exception $e)
        {
            return [
                'title' => $title,
                'description' => $description,
                'status_code' => $statusCode,
            ];
        }

        return [
            'title' => $title,
            'description' => $description
        ];
    }

    protected function getCharset($header, $html)
    {
        $charset = null;

        if(!empty($header))
        {
            if (isset($parts[1]) && preg_match('/charset=([-a-z0-9_]+)/i', $header, $match))
            {
                $charset = $match[1];
            }
        }

        preg_match('#<meta(?!\s*(?:name|value)\s*=)(?:[^>]*?content\s*=[\s"\']*)?([^>]*?)[\s"\';]*charset\s*=[\s"\']*([^\s"\'\/>]*)#i', $html, $contentTypeMatch);

        if (!empty($contentTypeMatch[2]))
        {
            $charset = trim($contentTypeMatch[2], " \t\n\r\0\x0B\"");
        }

        if ($charset == null)
        {
            $charset = 'iso-8859-1';
        }


        return $charset;
    }

    protected function getTags($html)
    {
        if(preg_match('#<meta[^>]+name=("|\')keywords("|\')[^>]*content="([^">]+)"#simU', $html, $match))
        {
            $tags = str_replace('.', '', $match[3]);
            $tags = trim($this->cleanUpPageInfos($tags, $this->charset));

            return $tags;
        }

        return '';
    }

    protected function getTitle($html)
    {
        $title = null;

        if(preg_match('#<meta[^>]+property="(og:|twitter:)title"[^>]*content="([^">]+)"#simU', $html, $match))
        {
            $title = isset($match[2]) ? $match[2] : '';
        }

        if($title === null && preg_match('#<title[^>]*>(.*)</title>#siU', $html, $match))
        {
            $title = $match[1];
        }

        $title = str_replace("\n", ' ', trim($title));
        return trim($this->cleanUpPageInfos($title, $this->charset));
    }

    protected function getDescription($html)
    {
        if(preg_match('#<meta[^>]+name=("|\')description("|\')[^>]*content="([^">]+)"#simU', $html, $match))
        {
            if(!empty($match[3]))
            {
                return trim($this->cleanUpPageInfos($match[3], $this->charset));
            }
        }

        return '';
    }

    private function cleanUpPageInfos($text, $charset)
    {
        $text = \XF::cleanString($this->_toUtf8($text, $charset, false));
        if(defined('ENT_HTML5'))
        {
            $text = html_entity_decode($text, ENT_QUOTES | ENT_HTML5, 'UTF-8');
        }
        else
        {
            $text = html_entity_decode($text, ENT_QUOTES, 'UTF-8');
        }

        return utf8_unhtml($text);
    }


    protected function _toUtf8($string, $charset, $entities = null)
    {
        // note: assumes charset is ascii compatible
        if (preg_match('/[\x80-\xff]/', $string))
        {
            $newString = false;
            if (function_exists('iconv'))
            {
                $newString = @iconv($charset, 'utf-8//IGNORE', $string);
            }
            if (!$newString && function_exists('mb_convert_encoding'))
            {
                $newString = @mb_convert_encoding($string, 'utf-8', $charset);
            }
            $string = ($newString ? $newString : preg_replace('/[\x80-\xff]/', '', $string));
        }

        $string = utf8_unhtml($string, $entities);
        $string = preg_replace('/[\xF0-\xF7].../', '', $string);
        $string = preg_replace('/[\xF8-\xFB]..../', '', $string);
        return $string;
    }

    protected function getcUrlHeaders($response)
    {
        $headers = array();

        $header_text = substr($response, 0, strpos($response, "\r\n\r\n"));

        foreach(@explode("\r\n", $header_text) as $i => $line)
        {
            if ($i === 0)
            {
                $headers['http_code'] = $line;
            }
            else
            {
                @list($key, $value) = explode(': ', $line);
                $headers[$key] = $value;
            }
        }

        return $headers;
    }

}