<?php

namespace xenMade\LD\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class Link extends Entity
{
	const RATING_WEIGHTED_THRESHOLD = 10;
	const RATING_WEIGHTED_AVERAGE = 3;

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xenmade_ld_link';
        $structure->shortName = 'xenMade\LD:Link';
        $structure->primaryKey = 'link_id';
        $structure->contentType = 'ld_link';

        $structure->columns = [
            'link_id' => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],

            'title' => ['type' => self::STR, 'maxLength' => 100, 'required' => 'please_enter_valid_title', 'censor' => true],
            'description' => ['type' => self::STR, 'required' => 'ld_please_enter_valid_description', 'censor' => true],
            'url' => ['type' => self::STR, 'maxLength' => 2083, 'required' => 'ld_please_enter_valid_url', 'match' => 'url'],
            'affiliate_url' => ['type' => self::STR, 'maxLength' => 2083, 'match' => 'url_empty', 'default' => ''],
            'backlink_location' => ['type' => self::STR, 'maxLength' => 2083, 'match' => 'url_empty', 'default' => ''],

            'user_id' => ['type' => self::UINT, 'required' => true],
            'username' => ['type' => self::STR, 'maxLength' => 50, 'required' => 'please_enter_valid_name'],

            'state' => ['type' => self::STR, 'default' => 'visible', 'allowedValues' => ['visible', 'moderated', 'deleted']],

            'insert_date' => ['type' => self::UINT, 'default' => \XF::$time],
            'last_update' => ['type' => self::UINT, 'default' => 0],
            'image_date' => ['type' => self::UINT, 'default' => 0],
            'backlink_check' => ['type' => self::UINT, 'default' => 0],
            'status_check' => ['type' => self::UINT, 'default' => 0],
            'status_fails' => ['type' => self::UINT, 'default' => 0],

            'warning_id' => ['type' => self::UINT, 'default' => 0],
            'warning_message' => ['type' => self::STR, 'maxLength' => 255, 'default' => '', 'censor' => true],

            'ip_id' => ['type' => self::UINT, 'default' => 0],

            'category_id' => ['type' => self::UINT, 'required' => true],
            'thread_id' => ['type' => self::UINT, 'default' => 0],

            'click_count' => ['type' => self::UINT, 'default' => 0, 'forced' => true],
            'comment_count' => ['type' => self::UINT, 'default' => 0, 'forced' => true],

            'rating_count' => ['type' => self::UINT, 'default' => 0, 'forced' => true],
            'rating_sum' => ['type' => self::UINT, 'default' => 0, 'forced' => true],
            'rating_avg' => ['type' => self::FLOAT, 'default' => 0],
            'rating_weighted' => ['type' => self::FLOAT, 'default' => 0],

            'custom_fields' => ['type' => self::SERIALIZED_ARRAY, 'default' => []],
            'tags' => ['type' => self::SERIALIZED_ARRAY, 'default' => []],

            'prefix_id' => ['type' => self::UINT, 'default' => 0],

            'is_vip' => ['type' => self::UINT, 'default' => 0],

            'likes' => ['type' => self::UINT, 'forced' => true, 'default' => 0],
            'like_users' => ['type' => self::SERIALIZED_ARRAY, 'default' => []],
        ];

        $structure->getters = [
            'custom_fields' => true,
            'rating_ids' => true,
            'link_title' => true
        ];

        $structure->behaviors = [
            'XF:Likeable' => ['stateField' => 'state'],

            'XF:Taggable' => ['stateField' => 'state'],

            'XF:Indexable' => [
                'checkForUpdates' => ['title', 'category_id', 'user_id', 'prefix_id', 'tags', 'state']
            ],
            'XF:IndexableContainer' => [
                'childContentType' => 'ld_link',
                'childIds' => function($link) { return $link->link_id; },
                'checkForUpdates' => ['category_id', 'state', 'prefix_id']
            ],

            'XF:NewsFeedPublishable' => [
                'usernameField' => 'username',
                'dateField' => 'insert_date'
            ],

            'XF:CustomFieldsHolder' => [
                'valueTable' => 'xf_xenmade_ld_link_field_value',
                'checkForUpdates' => ['category_id'],
                'getAllowedFields' => function($link) { return $link->Category ? $link->Category->field_cache : []; }
            ]
        ];

        $structure->relations = [
            'Category' => [
                'entity' => 'xenMade\LD:Category',
                'type' => self::TO_ONE,
                'conditions' => 'category_id',
                'primary' => true
            ],
            'User' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => 'user_id',
                'primary' => true
            ],
            'Rating' => [
                'entity' => 'xenMade\LD:LinkRating',
                'type' => self::TO_MANY,
                'conditions' => 'link_id',
                'key' => 'user_id'
            ],
            'Prefix' => [
                'entity' => 'xenMade\LD:LinkPrefix',
                'type' => self::TO_ONE,
                'conditions' => 'prefix_id',
                'primary' => true
            ],
            'Watch' => [
                'entity' => 'xenMade\LD:LinkWatch',
                'type' => self::TO_MANY,
                'conditions' => 'link_id',
                'key' => 'user_id'
            ],
            'DeletionLog' => [
                'entity' => 'XF:DeletionLog',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['content_type', '=', 'ld_link'],
                    ['content_id', '=', '$link_id']
                ],
                'primary' => true
            ],
            'ApprovalQueue' => [
                'entity' => 'XF:ApprovalQueue',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['content_type', '=', 'ld_link'],
                    ['content_id', '=', '$link_id']
                ],
                'primary' => true
            ],
            'Likes' => [
                'entity' => 'XF:LikedContent',
                'type' => self::TO_MANY,
                'conditions' => [
                    ['content_type', '=', 'ld_link'],
                    ['content_id', '=', '$link_id']
                ],
                'key' => 'like_user_id',
                'order' => 'like_date'
            ],
            'Thread' => [
                'entity' => 'XF:Thread',
                'type' => self::TO_ONE,
                'conditions' => [['thread_id', '=', '$thread_id']],
                'primary' => true
            ],
        ];

        $structure->options = [
            'log_moderator' => true
        ];

        return $structure;
    }


    public function getBreadcrumbs($includeSelf = true)
    {
        $breadcrumbs = $this->Category ? $this->Category->getBreadcrumbs() : [];

        if ($includeSelf)
        {
            $breadcrumbs[] = [
                'href' => $this->app()->router()->buildLink('link-directory', $this),
                'value' => $this->title
            ];
        }

        return $breadcrumbs;
    }

    public function getFieldEditMode()
    {
        $visitor = \XF::visitor();

        $isSelf = ($visitor->user_id == $this->user_id || !$this->link_id);
        $isMod = ($visitor->user_id && $this->hasPermission('editAny'));

        if ($isMod || !$isSelf)
        {
            return $isSelf ? 'moderator_user' : 'moderator';
        }
        else
        {
            return 'user';
        }
    }

    public function getCustomFields()
    {
        /** @var \XF\CustomField\DefinitionSet $fieldDefinitions */
        $fieldDefinitions = $this->app()->container('customFields.ld_link');
        return new \XF\CustomField\Set($fieldDefinitions, $this);
    }

    public function softDelete($reason = '', \XF\Entity\User $byUser = null)
    {
        $byUser = $byUser ?: \XF::visitor();

        if ($this->state == 'deleted')
        {
            return false;
        }

        $this->state = 'deleted';

        /** @var \XF\Entity\DeletionLog $deletionLog */
        $deletionLog = $this->getRelationOrDefault('DeletionLog');
        $deletionLog->setFromUser($byUser);
        $deletionLog->delete_reason = $reason;

        $this->save();

        return true;
    }

    public function getImage($canonical = false)
    {
        if ($this->image_date)
        {
            return

                $this->app()->applyExternalDataUrl(
                    sprintf('ld_thumbnail/%d/%d.jpg?%d',
                        floor($this->link_id / 1000),
                        $this->link_id,
                        $this->image_date
                ), $canonical
            );
        }
        else
        {
            return null;
        }
    }

    public function getAbstractedThumnailPath($sizeCode = null)
    {
        return sprintf('data://ld_thumbnail/%s/%d.jpg',
            floor($this->link_id / 1000),
            $this->link_id
        );
    }

    public function isThumbnailAvailable()
    {
        $file = $this->getAbstractedThumnailPath();
        return $file && \XF::app()->fs()->has($file);
    }

    public function rebuildLastLink()
    {
        $link = $this->db()->fetchRow("
			SELECT *
			FROM xf_xenmade_ld_link
			WHERE category_id = ?
				AND state = 'visible'
			ORDER BY last_update DESC
			LIMIT 1
		", $this->category_id);

        if ($link)
        {
            $this->last_update = $link['last_update'];
            $this->last_link_title = $link['title'];
            $this->last_link_id = $link['link_id'];
        }
        else
        {
            $this->last_update = 0;
            $this->last_link_title = '';
            $this->last_link_id = 0;
        }
    }

    public function getRatingIds()
    {
        return $this->db()->fetchAllColumn("
			SELECT rating_id
			FROM xf_xenmade_ld_link_rating
			WHERE link_id = ?
			ORDER BY rating_date
		", $this->link_id);
    }

    public function rebuildRating()
    {
        $rating = $this->db()->fetchRow("
			SELECT COUNT(*) AS total,
				SUM(rating) AS sum
			FROM xf_xenmade_ld_link_rating
			WHERE link_id = ?
				AND rating_state = 'visible'
		", $this->link_id);

        $this->rating_sum = $rating['sum'] ?: 0;
        $this->rating_count = $rating['total'] ?: 0;
    }

    public function rebuildComments()
    {
        $comments = $this->db()->fetchRow("
			SELECT COUNT(*) AS total
			FROM xf_xenmade_ld_link_comment
			WHERE link_id = ?
				AND message_state = 'visible'
		", $this->link_id);

        $this->comment_count = $comments['total'] ? : 0;
    }

    public function rebuildLikes()
    {
        $comments = $this->db()->fetchRow("
			SELECT COUNT(*) AS total
			FROM xf_liked_content
			WHERE content_id = ?
				AND content_type = 'ld_link'
		", $this->link_id);

        $this->likes = $comments['total'] ? : 0;
    }

    public function hasPermission($permission)
    {
        return \XF::visitor()->hasLinkCategoryPermission($this->category_id, $permission);
    }

    public function isVisible()
    {
        return ($this->state == 'visible');
    }

    public function isIgnored()
    {
        return \XF::visitor()->isIgnoring($this->user_id);
    }

    public function canView(&$error = null)
    {
        if (!$this->Category || !$this->Category->canView())
        {
            return false;
        }

        if (!$this->hasPermission('view'))
        {
            return false;
        }

        if ($this->state == 'moderated')
        {
            if (
                !$this->hasPermission('viewModerated')
                && (!\XF::visitor()->user_id || \XF::visitor()->user_id != $this->user_id)
            )
            {
                return false;
            }
        }
        else if ($this->state == 'deleted')
        {
            if (!$this->hasPermission('viewDeleted'))
            {
                return false;
            }
        }

        return true;
    }

    public function canUseInlineModeration(&$error = null)
    {
        return (\XF::visitor()->user_id && $this->hasPermission('inlineMod'));
    }

    public function canDelete($type = 'soft', &$error = null)
    {
        if ($type != 'soft')
        {
            return $this->hasPermission('hardDeleteAny');
        }

        if ($this->hasPermission('deleteAny'))
        {
            return true;
        }

        return ($this->user_id == \XF::visitor()->user_id && $this->hasPermission('deleteOwn'));
    }

    public function canUndelete(&$error = null)
    {
        return \XF::visitor()->user_id && $this->hasPermission('undelete');
    }

    public function canApproveUnapprove(&$error = null)
    {
        return (\XF::visitor()->user_id && $this->hasPermission('approveUnapprove'));
    }

    public function canReassign(&$error = null)
    {
        return (\XF::visitor()->user_id && $this->hasPermission('reassign'));
    }

    public function canMove(&$error = null)
    {
        return (\XF::visitor()->user_id && $this->hasPermission('editAny'));
    }

    public function canEdit(&$error = null)
    {
        if (!\XF::visitor()->user_id)
        {
            return false;
        }

        if ($this->hasPermission('editAny'))
        {
            return true;
        }

        return ($this->user_id == \XF::visitor()->user_id && $this->hasPermission('updateOwn'));
    }

    public function canVIP(&$error = null)
    {
        if (!\XF::visitor()->user_id)
        {
            return false;
        }

        return $this->hasPermission('canVIPLink');
    }

    public function canClaim(&$error = null)
    {
        if (\XF::visitor()->user_id && \XF::visitor()->user_id != $this->user_id)
        {
            return true;
        }

        return false;
    }

    public function canEditAny(&$error = null)
    {
        return (\XF::visitor()->user_id && $this->hasPermission('editAny'));
    }

    public function canLike(&$error = null)
    {
        $visitor = \XF::visitor();
        if (!$visitor->user_id)
        {
            return false;
        }

        if ($this->state != 'visible')
        {
            return false;
        }

        if ($this->user_id == $visitor->user_id)
        {
            $error = \XF::phraseDeferred('liking_own_content_cheating');
            return false;
        }

        return $visitor->hasPermission('ld_link', 'like');
    }

    public function isLiked()
    {
        $visitor = \XF::visitor();
        if (!$visitor->user_id)
        {
            return false;
        }

        return isset($this->Likes[$visitor->user_id]);
    }

    public function canSetAffiliateLink(&$error = null)
    {
        if (!\XF::visitor()->user_id)
        {
            return false;
        }

        return $this->hasPermission('setAffiliateLink');
    }

    public function canEditTags(&$error = null)
    {
        $category = $this->Category;
        return $category ? $category->canEditTags($this, $error) : false;
    }

    public function canViewRatings(&$error = null)
    {
        if (!$this->isVisible())
        {
            return false;
        }

        return $this->hasPermission('viewRatings');
    }

    public function canRate(&$error = null)
    {
        if (!$this->isVisible())
        {
            return false;
        }

        if(!$this->hasPermission('viewRatings'))
        {
            return false;
        }

        $visitor = \XF::visitor();

        if (!$visitor->user_id)
        {
            return false;
        }

        if ($visitor->user_id == $this->user_id)
        {
            return $this->hasPermission('rateOwn');
        }

        return $this->hasPermission('rate');
    }

    public function canAddComment(&$error = null)
    {
        if (!$this->isVisible())
        {
            return false;
        }

        return \XF::visitor()->hasPermission('ld_link_comment', 'post');
    }

    public function canViewDeletedComments(&$error = null)
    {
        if (!$this->isVisible())
        {
            return false;
        }

        return \XF::visitor()->hasPermission('ld_link_comment', 'viewModerated');
    }

    public function canViewDeletedRatings(&$error = null)
    {
        if (!$this->isVisible())
        {
            return false;
        }

        return \XF::visitor()->hasPermission('ld_link', 'viewModerated');
    }

    public function canViewModeratedComments(&$error = null)
    {
        if (!$this->isVisible())
        {
            return false;
        }

        return true;
    }

    public function canReport(&$error = null, \XF\Entity\User $user = null)
    {
        $user = $user ?: \XF::visitor();
        return $user->canReport($error);
    }

    public function canWarn(&$error = null)
    {
        $visitor = \XF::visitor();

        if ($this->warning_id
            || !$this->link_id
            || !$this->user_id
            || !$visitor->user_id
            || $this->user_id == $visitor->user_id
            || !$this->hasPermission('warn')
        )
        {
            return false;
        }

        return true;
    }

    public function canWatch(&$error = null)
    {
        $visitor = \XF::visitor();

        return ($visitor->user_id && $visitor->user_id != $this->user_id);
    }

    public function rebuildCounters()
    {
        $this->rebuildRating();
        $this->rebuildComments();
        $this->rebuildLikes();
        return true;
    }

    public function getNewLinkComment()
    {
        $linkComment = $this->_em->create('xenMade\LD:LinkComment');
        $linkComment->link_id = $this->link_id;

        return $linkComment;
    }

    public function getAffiliateUrl()
    {
        return 'onclick="window.open(\'' . $this->affiliate_url . '\',\'_blank\'); return false;"';
    }

    public function getLinkTitle()
    {
        return $this->title ? $this->title : '';
    }




    protected function _preSave()
    {
        if ($this->prefix_id && $this->isChanged(['prefix_id', 'category_id']))
        {
            if (!$this->Category->isPrefixValid($this->prefix_id))
            {
                $this->prefix_id = 0;
            }
        }

        if ($this->isInsert() || $this->isChanged(['rating_sum', 'rating_count']))
        {
            if($this->getValue('rating_sum'))
            {
                $this->updateRatingAverage();
            }
        }
    }

    protected function _postSave()
    {
        $visibilityChange = $this->isStateChanged('state', 'visible');
        $approvalChange = $this->isStateChanged('state', 'moderated');
        $deletionChange = $this->isStateChanged('state', 'deleted');

        if ($this->isUpdate())
        {
            if ($visibilityChange == 'enter')
            {
                $this->madeVisible();

                if ($approvalChange)
                {
                    $this->submitHamData();
                }
            }
            else if ($visibilityChange == 'leave')
            {
                $this->linkHidden();
            }

            if ($deletionChange == 'leave' && $this->DeletionLog)
            {
                $this->DeletionLog->delete();
            }

            if ($approvalChange == 'leave' && $this->ApprovalQueue)
            {
                $this->ApprovalQueue->delete();
            }
        }
        else
        {
            if ($this->state == 'visible')
            {
                $this->insertedVisible();
            }
        }

        if ($this->isUpdate())
        {
            if ($this->isChanged('user_id'))
            {
                $this->linkReassigned();
            }


            if ($this->thread_id && $visibilityChange == 'enter')
            {

                if ($this->Thread && $this->Thread->discussion_type == 'ld_link')
                {
                    $this->Thread->title = $this->getLinkTitle();
                    $this->Thread->saveIfChanged($saved, false, false);
                }
            }

        }

        if ($approvalChange == 'enter')
        {
            $approvalQueue = $this->getRelationOrDefault('ApprovalQueue', false);
            $approvalQueue->content_date = $this->insert_date;
            $approvalQueue->save();
        }
        else if ($deletionChange == 'enter' && !$this->DeletionLog)
        {
            $delLog = $this->getRelationOrDefault('DeletionLog', false);
            $delLog->setFromVisitor();
            $delLog->save();
        }

        $this->updateCategoryRecord();

        if ($this->isUpdate() && $this->getOption('log_moderator'))
        {
            $this->app()->logger()->logModeratorChanges('ld_link', $this);
        }
    }

    protected function _postDelete()
    {
        if ($this->state == 'visible')
        {
            $this->linkHidden(true);
        }

        if ($this->Category && $this->state == 'visible')
        {
            $this->Category->linkRemoved($this);
            $this->Category->save();
        }

        if ($this->state == 'deleted' && $this->DeletionLog)
        {
            $this->DeletionLog->delete();
        }

        if ($this->state == 'moderated' && $this->ApprovalQueue)
        {
            $this->ApprovalQueue->delete();
        }

        if ($this->getOption('log_moderator'))
        {
            $this->app()->logger()->logModeratorAction('ld_link', $this, 'delete_hard');
        }

        $db = $this->db();

        if ($this->rating_ids)
        {
            $db->delete('xf_xenmade_ld_link_rating', 'rating_id IN (' . $db->quote($this->rating_ids) . ')');
        }

        $db->delete('xf_xenmade_ld_link_watch', 'link_id = ?', $this->link_id);


        if ($this->image_date)
        {
            \XF\Util\File::deleteFromAbstractedPath($this->getAbstractedThumnailPath());
        }
    }

    protected function updateCategoryRecord()
    {
        if (!$this->Category)
        {
            return;
        }

        $category = $this->Category;

        if ($this->isUpdate() && $this->isChanged('category_id'))
        {
            if ($this->state == 'visible')
            {
                $category->linkAdded($this);
                $category->save();
            }

            if ($this->getExistingValue('state') == 'visible')
            {
                /** @var Category $oldCategory */
                $oldCategory = $this->getExistingRelation('Category');
                if ($oldCategory)
                {
                    $oldCategory->linkRemoved($this);
                    $oldCategory->save();
                }
            }

            if ($this->thread_id && $this->Thread && $this->Thread->discussion_type == 'ld_link')
            {
                $thread = $this->Thread;

                if ($category->thread_node_id)
                {
                    $thread->node_id = $category->thread_node_id;
                    $thread->prefix_id = $category->thread_prefix_id;

                    if ($this->state == 'visible' && $thread->state == 'deleted')
                    {
                        $thread->discussion_state = 'visible';
                    }
                }
                else
                {
                    $thread->discussion_state = 'deleted';
                }

                $thread->saveIfChanged($saved, false, false);
            }

            return;
        }

        $visibilityChange = $this->isStateChanged('state', 'visible');
        if ($visibilityChange == 'enter' && $category)
        {
            $category->linkAdded($this);
            $category->save();
        }
        else if ($visibilityChange == 'leave' && $category)
        {
            $category->linkRemoved($this);
            $category->save();
        }
        else if ($this->isUpdate() && $this->state == 'visible')
        {
            $category->dataChanged($this);
            $category->save();
        }
    }

    protected function linkReassigned()
    {
        if ($this->state == 'visible')
        {
            $this->adjustUserLinkCountIfNeeded(-1, $this->getExistingValue('user_id'));
            $this->adjustUserLinkCountIfNeeded(1);
        }
    }

    protected function insertedVisible()
    {
        $this->adjustUserLinkCountIfNeeded(1);
    }

    protected function linkHidden($hardDelete = false)
    {
        $this->adjustUserLinkCountIfNeeded(-1);

        /** @var \XF\Repository\UserAlert $alertRepo */
        $alertRepo = $this->repository('XF:UserAlert');
        $alertRepo->fastDeleteAlertsForContent('ld_link_rating', $this->rating_ids);

        if (!$hardDelete)
        {
            /** @var \XF\Repository\LikedContent $likeRepo */
            $likeRepo = $this->repository('XF:LikedContent');
            $likeRepo->fastUpdateLikeIsCounted('ld_link', $this->link_id, false);
        }

        if ($this->thread_id && $this->Thread && $this->Thread->discussion_type == 'ld_link')
        {
            $thread = $this->Thread;

            switch ($this->app()->options()->ldLinkThreadDelete)
            {
                case 'delete':
                    $thread->discussion_state = 'deleted';
                    break;

                case 'close':
                    $thread->discussion_open = false;
                    break;
            }

            $thread->title = $this->getLinkTitle();

            try
            {
                $thread->saveIfChanged($saved, false, false);
            }
            catch (\Exception $e)
            {}
        }
    }

    protected function submitHamData()
    {
        /** @var \XF\Spam\ContentChecker $submitter */
        $submitter = $this->app()->container('spam.contentHamSubmitter');
        $submitter->submitHam('ld_link', $this->link_id);
    }

    protected function madeVisible()
    {
        $this->adjustUserLinkCountIfNeeded(1);

        /** @var \XF\Repository\LikedContent $likeRepo */
        $likeRepo = $this->repository('XF:LikedContent');
        $likeRepo->recalculateLikeIsCounted('ld_link', $this->link_id);

        if ($this->thread_id && $this->Thread && $this->Thread->discussion_type == 'ld_link')
        {
            $thread = $this->Thread;

            switch ($this->app()->options()->ldLinkThreadDelete)
            {
                case 'delete':
                    $thread->discussion_state = 'visible';
                    break;

                case 'close':
                    $thread->discussion_open = true;
                    break;
            }

            $thread->title = $this->getLinkTitle();
            $thread->saveIfChanged($saved, false, false);
        }
    }

    protected function adjustUserLinkCountIfNeeded($amount, $userId = null)
    {
        if ($userId === null)
        {
            $userId = $this->user_id;
        }

        if ($userId)
        {
            $this->db()->query("
				UPDATE xf_user
				SET ld_link_count = GREATEST(0, ld_link_count + ?)
				WHERE user_id = ?
			", [$amount, $userId]);
        }
    }

    protected function updateRatingAverage()
    {
        $threshold = self::RATING_WEIGHTED_THRESHOLD;
        $average = self::RATING_WEIGHTED_AVERAGE;

        $this->rating_weighted = ($threshold * $average + $this->rating_sum) / ($threshold + $this->rating_count);

        if ($this->rating_count)
        {
            $this->rating_avg = $this->rating_sum / $this->rating_count;
        }
        else
        {
            $this->rating_avg = 0;
        }
    }

    public function ratingAdded($rating)
    {
        $this->rating_count++;
        $this->rating_sum += $rating->rating;
    }

    public function canSendModeratorActionAlert()
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id || $visitor->user_id == $this->user_id)
        {
            return false;
        }

        if ($this->state != 'visible')
        {
            return false;
        }

        return (
            $visitor->hasPermission('ld_link', 'deleteAny')
            || $visitor->hasPermission('ld_link', 'editAny')
        );
    }

    public function hasThread()
    {
        if (!$this->thread_id)
        {
            return false;
        }

        $thread = $this->Thread;

        if (!$thread || !$thread->canView())
        {
            return false;
        }

        return true;
    }


    // CommentUpdater
    public function commentAdded()
    {
        $this->comment_count++;
    }

    public function commentRemoved()
    {
        $this->comment_count--;
    }

    //RatingUpdater
    public function ratingRemoved($rating)
    {
        $this->rating_count--;
        $this->rating_sum -= $rating->rating;
    }


}