<?php

namespace xenMade\LD\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class LinkUpgrade extends Entity
{
	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_xenmade_ld_link_upgrade';
		$structure->shortName = 'xenMade\LD:LinkUpgrade';
		$structure->primaryKey = 'upgrade_id';

		$structure->columns = [
			'upgrade_id' => ['type' => self::UINT, 'autoIncrement' => true],
			'can_purchase' => ['type' => self::BOOL, 'default' => 1],

			'title' => ['type' => self::STR, 'default' => '', 'required' => true],
			'description' => ['type' => self::STR, 'default' => ''],

			'display_order' => ['type' => self::UINT, 'default' => 20],

            'recurring' => ['type' => self::BOOL, 'default' => false],

            'cost_amount' => ['type' => self::FLOAT, 'required' => true, 'min' => 0],
            'cost_currency' => ['type' => self::STR, 'default' => 'USD'],

            'length_amount' => ['type' => self::UINT, 'max' => 365],
            'length_unit' => ['type' => self::STR, 'default' => '',
                'allowedValues' => ['day', 'month', 'year', '']
            ],

            'payment_profile_ids' => ['type' => self::LIST_COMMA,
                'required' => 'please_select_at_least_one_payment_profile',
                'list' => ['type' => 'posint', 'unique' => true, 'sort' => SORT_NUMERIC]
            ]
		];

        $structure->getters = [
            'cost_phrase' => true,
            'purchasable_type_id' => true
        ];

        $structure->relations = [
            'CategoryUpgrades' => [
                'entity' => 'xenMade\LD:CategoryUpgrade',
                'type' => self::TO_MANY,
                'conditions' => 'upgrade_id',
            ],

            'Category' => [
                'entity' => 'xenMade\LD:CategoryUpgrade',
                'type' => self::TO_ONE,
                'conditions' => 'upgrade_id',
            ],

            'Active' => [
                'entity' => 'xenMade\LD:LinkUpgradeActive',
                'type' => self::TO_MANY,
                'conditions' => 'upgrade_id',
                'key' => 'link_id'
            ],
        ];

		return $structure;
	}

    public function getPurchasableTypeId()
    {
        return 'ld_link_upgrade';
    }

    public function canPurchase(\xenMade\LD\Entity\Link $link)
    {
        return ($this->can_purchase && !isset($this->Active[$link->link_id]));
    }

    public function getCostPhrase()
    {
        $cost = $this->app()->data('XF:Currency')->languageFormat($this->cost_amount, $this->cost_currency);
        $phrase = $cost;

        if ($this->length_unit)
        {
            if ($this->length_amount > 1)
            {
                if ($this->recurring)
                {
                    $phrase = \XF::phrase("x_per_y_{$this->length_unit}s", [
                        'cost' => $cost,
                        'length' => $this->length_amount
                    ]);
                }
                else
                {
                    $phrase = \XF::phrase("x_for_y_{$this->length_unit}s", [
                        'cost' => $cost,
                        'length' => $this->length_amount
                    ]);
                }
            }
            else
            {
                if ($this->recurring)
                {
                    $phrase = \XF::phrase("x_per_{$this->length_unit}", [
                        'cost' => $cost
                    ]);
                }
                else
                {
                    $phrase = \XF::phrase("x_for_one_{$this->length_unit}", [
                        'cost' => $cost
                    ]);
                }
            }
        }

        return $phrase;
    }
}