<?php

namespace xenMade\LD\FindNew;

use XF\Entity\FindNew;
use XF\FindNew\AbstractHandler;

class Link extends AbstractHandler
{
	public function getRoute()
	{
		return 'whats-new/link-directory';
	}

	public function getPageReply(\XF\Mvc\Controller $controller, FindNew $findNew, array $results, $page, $perPage)
	{
		$canInlineMod = false;

		/** @var \xenMade\LD\Entity\Link $link */
		foreach ($results AS $link)
		{
			if ($link->canUseInlineModeration())
			{
				$canInlineMod = true;
				break;
			}
		}

		$viewParams = [
			'findNew' => $findNew,

			'page' => $page,
			'perPage' => $perPage,

			'links' => $results,
			'canInlineMod' => $canInlineMod
		];
		return $controller->view('xenMade\LD:WhatsNew\Links', 'ld_whats_new_links', $viewParams);
	}

	public function getFiltersFromInput(\XF\Http\Request $request)
	{
		$filters = [];

		$visitor = \XF::visitor();

		$watched = $request->filter('watched', 'bool');
		if ($watched && $visitor->user_id)
		{
			$filters['watched'] = true;
		}

		return $filters;
	}

	public function getDefaultFilters()
	{
		return [];
	}

	public function getResultIds(array $filters, $maxResults)
	{
		$visitor = \XF::visitor();

		/** @var \xenMade\LD\Finder\Link $finder */
		$finder = \XF::finder('xenMade\LD:Link')
			->with('Category', true)
			->with('Category.Permissions|' . $visitor->permission_combination_id)
			->where('state', '<>', 'deleted')
			->where('last_update', '>', \XF::$time - (86400 * \XF::options()->readMarkingDataLifetime))
			->order('last_update', 'DESC');

		$this->applyFilters($finder, $filters);

        $links = $finder->fetch($maxResults);
        $links = $this->filterResults($links);

		return $links->keys();
	}

	public function getPageResultsEntities(array $ids)
	{
		$visitor = \XF::visitor();

		$ids = array_map('intval', $ids);

		/** @var \xenMade\LD\Finder\Link $finder */
		$finder = \XF::finder('xenMade\LD:Link')
			->where('link_id', $ids)
			->forFullView(true)
			->with('Category.Permissions|' . $visitor->permission_combination_id);

		return $finder->fetch();
	}

	protected function filterResults(\XF\Mvc\Entity\ArrayCollection $results)
	{
		$visitor = \XF::visitor();

		return $results->filter(function(\xenMade\LD\Entity\Link $link) use($visitor)
		{
			return ($link->canView() && !$visitor->isIgnoring($link->user_id));
		});
	}

	protected function applyFilters(\xenMade\LD\Finder\Link $finder, array $filters)
	{
		$visitor = \XF::visitor();
		if (!empty($filters['watched']))
		{
			$finder->watchedOnly($visitor->user_id);
		}
	}

	public function getResultsPerPage()
	{
		return \XF::options()->ldLinksPerPage;
	}

	public function isAvailable()
	{
		/** @var \xenMade\LD\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		return $visitor->canViewLinks();
	}
}