<?php

namespace xenMade\LD\Finder;

use XF\Mvc\Entity\Finder;

class Link extends Finder
{
	public function applyGlobalVisibilityChecks($allowOwnPending = false)
	{
		$visitor = \XF::visitor();
		$conditions = [];
		$viewableStates = ['visible'];

		if ($visitor->hasPermission('ld_link', 'viewDeleted'))
		{
			$viewableStates[] = 'deleted';

			$this->with('DeletionLog');
		}

		if ($visitor->hasPermission('ld_link', 'viewModerated'))
		{
			$viewableStates[] = 'moderated';
		}
		else if ($visitor->user_id && $allowOwnPending)
		{
			$conditions[] = [
				'state' => 'moderated',
				'user_id' => $visitor->user_id
			];
		}

		$conditions[] = ['state', $viewableStates];

		$this->whereOr($conditions);

		return $this;
	}

	public function watchedOnly($userId = null)
	{
		if ($userId === null)
		{
			$userId = \XF::visitor()->user_id;
		}
		if (!$userId)
		{
			return $this;
		}

		$this->whereOr(
			['Watch|' . $userId . '.user_id', '!=', null],
			['Category.Watch|' . $userId . '.user_id', '!=', null]
		);

		return $this;
	}

	public function forFullView($includeCategory = true)
	{
		$visitor = \XF::visitor();

		$this->with(['User']);

		if ($visitor->user_id)
		{
			$this->with('Watch|' . $visitor->user_id);
		}

		if ($includeCategory)
		{
			$this->with(['Category']);

			if ($visitor->user_id)
			{
				$this->with('Category.Watch|' . $visitor->user_id);
			}
		}

		return $this;
	}

	public function useDefaultOrder()
	{
		$defaultOrder = $this->app()->options()->ldListDefaultOrder ?: 'insert_date';
		$defaultDir = $defaultOrder == 'title' ? 'asc' : 'desc';

		$this->setDefaultOrder($defaultOrder, $defaultDir);

		return $this;
	}
}