<?php

namespace xenMade\LD\Finder;

use XF\Mvc\Entity\Finder;

class LinkComment extends Finder
{
	public function forFullView()
	{
		$this->with('Link');

		$userId = \XF::visitor()->user_id;
		if ($userId)
		{
			$this->with('Likes|' . $userId);
		}

		return $this;
	}

	public function onLink(\xenMade\LD\Entity\Link $link, array $limits = [])
	{
		$limits = array_replace([
			'visibility' => true,
			'allowOwnPending' => true
		], $limits);

		$this->where('link_id', $link->link_id);

		if ($limits['visibility'])
		{
			$this->applyVisibilityChecksForLink($link, $limits['allowOwnPending']);
		}

		$this->forFullView();

		return $this;
	}

	public function applyVisibilityChecksForLink(\xenMade\LD\Entity\Link $link, $allowOwnPending = true)
	{
		$conditions = [];
		$viewableStates = ['visible'];

        $visitor = \XF::visitor();

		if ($link->canViewDeletedComments())
		{
			$viewableStates[] = 'deleted';
			$this->with('DeletionLog');
		}

		if ($link->canViewModeratedComments())
		{
			$viewableStates[] = 'moderated';
		}
		else if ($visitor->user_id && $allowOwnPending)
		{
			$conditions[] = [
				'message_state' => 'moderated',
				'user_id' => $visitor->user_id
			];
		}

		$conditions[] = ['message_state', $viewableStates];

		$this->whereOr($conditions);

		return $this;
	}

	public function newerThan($date)
	{
		$this->where('post_date', '>', $date);

		return $this;
	}
}