<?php

namespace xenMade\LD\Import\Data;

use XF\Import\Data\AbstractEmulatedData;
use XF\Import\Data\HasDeletionLogTrait;

class LinkLikes extends AbstractEmulatedData
{
    public function getImportType()
    {
        return 'ld_link_likes';
    }

    public function getEntityShortName()
    {
        return 'XF:LikedContent';
    }

    protected function postSave($oldId, $newId)
    {
        if ($this->content_user_id)
        {
            $this->db()->query("
				UPDATE xf_user
				SET like_count = like_count + 1
				WHERE user_id = ?
			", $this->content_user_id);
        }

        /** @var \XF\Like\AbstractHandler $likeHandler */
        $likeHandler = $this->getLikeHandler('ld_link');

        if ($likeHandler)
        {
            $contentId = $this->content_id;

            $entity = $likeHandler->getContent($contentId);

            if ($entity)
            {
                $latest = [];

                $count = $this->db()->fetchOne("
                            SELECT COUNT(*)
                            FROM xf_liked_content
                            WHERE content_type = ? AND content_id = ?
                        ", ['ld_link', $contentId]);

                if ($count)
                {
                    $latest = $this->db()->fetchAll("
                            SELECT user.user_id, user.username
                            FROM xf_liked_content AS liked
                            INNER JOIN xf_user AS user ON (liked.like_user_id = user.user_id)
                            WHERE liked.content_type = ? AND liked.content_id = ?
                            ORDER BY liked.like_date DESC
                            LIMIT 5
                        ", ['ld_link', $contentId]);
                }
            }

            $likeHandler->updateContentLikes($entity, $count, $latest);
        }
    }

    protected function getLikeHandler($type, $throw = false)
    {
        $handlerClass = \XF::app()->getContentTypeFieldValue($type, 'like_handler_class');
        if (!$handlerClass)
        {
            return null;
        }

        if (!class_exists($handlerClass))
        {
            return null;
        }

        $handlerClass = \XF::extendClass($handlerClass);
        return new $handlerClass($type);
    }
}