<?php

namespace xenMade\LD\Job;

use \XF\Mvc\Entity\Entity;
use XF\Job\AbstractJob;

class Backlink extends AbstractJob
{

    public function run($maxRunTime)
	{
		$startTime = microtime(true);

		$linkFinder = $this->app->finder('xenMade\LD:Link');

		$links = $linkFinder
            ->with('Category')
            ->where('backlink_check', '<=', \XF::$time)
            ->order('backlink_check', 'ASC')
            ->limit(100)
            ->fetch();

		foreach ($links AS $link)
		{
		    /** @var \xenMade\LD\Entity\Category $category */
		    $category = $link->Category;

            /** @var \XF\Entity\User $user */
		    $user = $link->User;

		    /** @var \xenMade\LD\Service\Link\Backlink $backlink */
            $backlink = $this->app->service('xenMade\LD:Link\Backlink', $link);
            $backlink->jobChecker = true;

            if(!$backlink->backlinkChecker())
            {
                $link->set('state', 'moderated');
            }

            $link->set('backlink_check', \XF::$time + (86400 * 7));
            $link->save(false);

            //$this->sendMail($user);

            if (microtime(true) - $startTime >= $maxRunTime)
            {
                break;
            }
		}

		return $this->complete();
	}

	protected function sendMail($user)
    {
        $mail = \XF::mailer()->newMail();
        $mail->setToUser($user);
        $mail->setTemplate('ld_backlink_checker', ['user' => $user, 'delete_time' => \XF::$time]);
        $mail->queue();
    }

    public function getStatusMessage()
	{
		return \XF::phrase('ld_backlink_checker');
	}

	public function canCancel()
	{
		return;
	}

	public function canTriggerByChoice()
	{
		return;
	}
}