<?php

namespace xenMade\LD\Job;

use XF\Job\AbstractJob;

class InsertUrl extends AbstractJob
{
    protected $defaultData = [
        'urls' => [],
        'fail_urls' => [],
        'url' => '',
        'count' => 0,
        'category_id' => 0,
    ];


    public function run($maxRunTime)
    {
        $urls = $this->data['urls'];

        if(count($urls) == 0)
        {
            return $this->complete();
        }

        $key = key($urls);
        $url = trim($urls[$key]);

        unset($urls[$key]);

        $this->data['urls'] = $urls;
        $this->data['url'] = $url;
        $this->data['count'] = count($urls);

        if (!$url)
        {
            return $this->complete();
        }

        $this->executeAction($url);

        return $this->resume();
    }

    protected function executeAction($url)
    {
        /** @var \xenMade\LD\Entity\Category $category */
        $category = $this->app->em()->find('xenMade\LD:Category', $this->data['category_id']);

        /** @var \xenMade\LD\Service\Link\WebData $webData */
        $webData = $this->app->service('xenMade\LD:Link\WebData');
        $data = $webData->fetchData($url);

        if($data['title'])
        {
            /** @var \xenMade\LD\Service\Link\Create $creator */
            $creator = $this->app->service('xenMade\LD:Link\Create', $category);

            $formatter = $this->app->stringFormatter();
            $data['title'] = trim($formatter->snippetString($data['title'], 100));

            $creator->setContent($data['title'], $data['description']);
            $creator->setUrl($data['url']);

            $creator->setWithOutDescription(true);

            if (!$creator->validate($errors))
            {
                //print_r($errors);
            }

            $link = $creator->save(false);

            if($link)
            {
                /** @var \xenMade\LD\Thumbnail\AbstractThumbnail $ldThumbnail */
                $ldThumbnail = \XF::app()->container('ldThumbnail');
                $ldThumbnail->setLink($link);
                $ldThumbnail->getThumbnail();

                $link->fastUpdate('image_date', \XF::$time + (86400 * 30));
            }
        }
        else
        {
            $this->data['fail_urls'][] = $url;

            \XF::app()->simpleCache()->LD->FailUrls = $this->data['fail_urls'];
        }

    }

    public function getStatusMessage()
    {
        if($this->data['url'])
        {
            return sprintf('%s: %s (%s)', \XF::phrase('ld_insert_url'), $this->data['url'], $this->data['count']);
        }

        return sprintf('%s', \XF::phrase('ld_mass_insert_starts_with_x_urls', ['count' => $this->data['count']]));
    }

    public function canCancel()
    {
        return true;
    }

    public function canTriggerByChoice()
    {
        return false;
    }
}