<?php

namespace xenMade\LD;

class Listener
{
    public static function app_setup(\XF\App $app)
    {
        $container = $app->container();

        $container['prefixes.ld_link'] = $app->fromRegistry('ldPrefixe',
            function(\XF\Container $c) { return $c['em']->getRepository('xenMade\LD:LinkPrefix')->rebuildPrefixCache(); }
        );

        $container['customFields.ld_link'] = $app->fromRegistry('ldField',
            function(\XF\Container $c)
            {
                return $c['em']->getRepository('xenMade\LD:LinkField')->rebuildFieldCache();

            },
            function(array $fields)
            {
                return new \XF\CustomField\DefinitionSet($fields);
            }
        );

        $container['ldThumbnail'] = function(\XF\Container $c) use($app)
        {
            $mode = $app->options()->ldThumbnail;

            if(!$mode)
            {
                $mode = 'upload';
            }

            $class = 'xenMade\LD\Thumbnail\\' . $mode;

            if (!$class || !class_exists($class))
            {
                $class = 'xenMade\LD\Thumbnail\Upload';
            }

            $class = $app->extendClass($class);
            return new $class($app);
        };
    }

    public static function controller_post_dispatch(\XF\Mvc\Controller $controller, $action, \XF\Mvc\ParameterBag $params, \XF\Mvc\Reply\AbstractReply &$reply)
    {
        if($reply instanceof \XF\Mvc\Reply\View)
        {
            $template = $reply->getTemplateName();

            /** @var \XF\Entity\Thread $thread */
            $thread = $reply->getParam('thread');

            if ($template == 'thread_view' && $thread && $thread->discussion_type == 'ld_link')
            {
                /** @var \xenMade\LD\Entity\Link $link */
                $link = \XF::repository('xenMade\LD:Link')->findLinkForThread($thread)->fetchOne();

                if ($link && $link->canView())
                {
                    $reply->setParam('link', $link);
                }
            }
        }
    }

    public static function criteria_user($rule, array $data, \XF\Entity\User $user, &$returnValue)
    {
        switch ($rule)
        {
            case 'ld_link_count':
                if (isset($user->ld_link_count) && $user->ld_link_count >= $data['links'])
                {
                    $returnValue = true;
                }
                break;
        }
    }

    public static function user_content_change_init(\XF\Service\User\ContentChange $changeService, array &$updates)
    {
        $updates['xf_xenmade_ld_category_watch'] = ['user_id', 'emptyable' => false];
        $updates['xf_xenmade_ld_link'] = ['user_id', 'username'];
        $updates['xf_xenmade_ld_link_rating'] = ['user_id', 'emptyable' => false];
        $updates['xf_xenmade_ld_link_watch'] = ['user_id', 'emptyable' => false];
        $updates['xf_xenmade_ld_link_comment'] = ['user_id', 'emptyable' => false];
    }

    public static function user_delete_clean_init(\XF\Service\User\DeleteCleanUp $deleteService, array &$deletes)
    {
        $deletes['xf_xenmade_ld_category_watch'] = 'user_id = ?';
        $deletes['xf_xenmade_ld_link_watch'] = 'user_id = ?';

        /*
        $deletes['xf_xenmade_ld_link'] = 'user_id = ?';
        $deletes['xf_xenmade_ld_link_rating'] = 'user_id = ?';
        $deletes['xf_xenmade_ld_link_comment'] = 'user_id = ?';
        */
    }

    public static function user_merge_combine(\XF\Entity\User $target, \XF\Entity\User $source, \XF\Service\User\Merge $mergeService)
    {
        $target->ld_link_count += $source->ld_link_count;
    }

    public static function user_searcher_orders(\XF\Searcher\User $userSearcher, array &$sortOrders)
    {
        $sortOrders['ld_link_count'] = \XF::phrase('ld_link_count');
    }

    public static function templater_setup(\XF\Container $container, \XF\Template\Templater &$templater)
    {
        $templater->addFunction('ld_thumbnail', [__CLASS__, 'ldfnThumbnail']);
        $templater->addFunction('ld_thumbnail_form', [__CLASS__, 'ldfnThumbnailForm']);
    }

    public static function ldfnThumbnail($templater, &$escape, \xenMade\LD\Entity\Link $link, $size = 'l')
    {
        $escape = false;

        if (!$link->image_date)
        {
            return "<span class=\"avatar avatar--linkImageDefault\"><span></span><span class=\"u-srOnly\">" . \XF::phrase('ld_link_image') . "</span></span>";
        }
        else
        {
            $src = $link->getImage();

            if (!$link->isThumbnailAvailable())
            {
                return "<span class=\"avatar avatar--linkImageDefault\"><span></span><span class=\"u-srOnly\">" . \XF::phrase('ld_link_image') . "</span></span>";
            }

            return "<span class=\"avatar avatar--{$size}\">" . '<img src="' . htmlspecialchars($src) . '" alt="' . htmlspecialchars($link->title) . '" /></span>';
        }
    }

    public static function ldfnThumbnailForm($templater, &$escape)
    {
        $thumbnail = \XF::app()->container('ldThumbnail');
        $escape = false;
        return $thumbnail->form($templater);
    }

    public static function templater_macro_pre_render(\XF\Template\Templater $templater, &$type, &$template, &$name, array &$arguments, array &$globalVars)
    {
    }

    public static function import_importer_classes(\XF\SubContainer\Import $container, \XF\Container $parentContainer, array &$importers)
    {
        $importers[] = 'xenMade\LD:LD';
    }

    public static function entity_post_delete(\XF\Mvc\Entity\Entity $entity)
    {
        if($entity instanceof \XF\Entity\Thread)
        {
            if(!empty($entity->thread_id) && $entity->discussion_type == 'ld_link')
            {
                \XF::db()->query('UPDATE xf_xenmade_ld_link SET thread_id = 0 WHERE thread_id = ?', $entity->thread_id);
            }
        }
    }
}