<?php

namespace xenMade\LD\ModeratorLog;

use XF\Entity\ModeratorLog;
use XF\ModeratorLog\AbstractHandler;
use XF\Mvc\Entity\Entity;

class LinkRating extends AbstractHandler
{
	public function isLoggable(Entity $content, $action, \XF\Entity\User $actor)
	{
	    return parent::isLoggable($content, $action, $actor);
	}

	protected function getLogActionForChange(Entity $content, $field, $newValue, $oldValue)
	{
		switch ($field)
		{
			case 'rating_state':
				if ($newValue == 'visible' && $oldValue == 'deleted')
				{
					return 'undelete';
				}
				else if ($newValue == 'deleted')
				{
					$reason = $content->DeletionLog ? $content->DeletionLog->delete_reason : '';
					return ['delete_soft', ['reason' => $reason]];
				}
				break;
		}

		return false;
	}

	protected function setupLogEntityContent(ModeratorLog $log, Entity $content)
	{
		/** @var \xenMade\LD\Entity\LinkRating $content */
		$link = $content->Link;

		$log->content_user_id = $content->user_id;
		$log->content_username = $content->User->username;
		$log->content_title = $link->title;
		$log->content_url = \XF::app()->router('public')->buildLink('nopath:link-directory/ratings', $content);
		$log->discussion_content_type = 'ld_link';
		$log->discussion_content_id = $content->link_id;
	}

	public function getContentTitle(ModeratorLog $log)
	{
		return \XF::phrase('ld_link_rating_in_x', ['title' => \XF::app()->stringFormatter()->censorText($log->content_title_)]);
	}
}