<?php

namespace xenMade\LD\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class LinkComment extends AbstractController
{
	public function actionIndex(ParameterBag $params)
	{
	    if(!$params->comment_id)
        {
            return $this->redirectPermanently($this->buildLink('link-directory'));
        }

		$linkComment = $this->assertViewableLinkComment($params->comment_id);

		if ($this->filter('_xfWithData', 'bool'))
		{
			$this->request->set('_xfDisableInlineMod', true);
			return $this->rerouteController(__CLASS__, 'show', $params);
		}

		$linkCommentRepo = $this->getLinkCommentRepo();

		$linkCommentFinder = $linkCommentRepo->findCommentsOnLink($linkComment->Link);
		$linkComments = $linkCommentFinder->where('post_date', '>', $linkComment->post_date)->fetch();

		$page = floor($linkComments->count() / $this->options()->messagesPerPage) + 1;

		return $this->redirectPermanently($this->buildLink('link-directory/comments', $linkComment->Link, ['page' => $page]) . '#link-comment-' . $linkComment->comment_id);
	}

	public function actionShow(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);

		$viewParams = [
			'linkComment' => $linkComment,
		];
		return $this->view('xenMade\LD:LinkComment\Show', 'ld_link_comment', $viewParams);
	}

    public function actionEdit(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);
		if (!$linkComment->canEdit($error))
		{
			return $this->noPermission($error);
		}

		$noInlineMod = $this->filter('_xfNoInlineMod', 'bool');

		if ($this->isPost())
		{
			$editor = $this->setupEdit($linkComment);
			$editor->checkForSpam();

			if (!$editor->validate($errors))
			{
				return $this->error($errors);
			}
			$editor->save();

			if ($this->filter('_xfWithData', 'bool') && $this->filter('_xfInlineEdit', 'bool'))
			{
				$viewParams = [
					'linkComment' => $linkComment,
					'noInlineMod' => $noInlineMod
				];

				$reply = $this->view('xenMade\LD:LinkComment\EditNewLinkComment', 'ld_link_comment_edit_new_comment', $viewParams);
				$reply->setJsonParam('message', \XF::phrase('your_changes_have_been_saved'));
				return $reply;
			}
			else
			{
				return $this->redirect($this->buildLink('link-directory/comment', $linkComment));
			}
		}
		else
		{
			$viewParams = [
				'linkComment' => $linkComment,
				'link' => $linkComment->Link,

				'quickEdit' => $this->filter('_xfWithData', 'bool'),
				'noInlineMod' => $noInlineMod
			];
			return $this->view('xenMade\LD:LinkComment\Edit', 'ld_link_comment_edit', $viewParams);
		}
	}

	public function actionDelete(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);
		if (!$linkComment->canDelete('soft', $error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			$type = $this->filter('hard_delete', 'bool') ? 'hard' : 'soft';
			$reason = $this->filter('reason', 'str');

			if (!$linkComment->canDelete($type, $error))
			{
				return $this->noPermission($error);
			}

			/** @var \xenMade\LD\Service\LinkComment\Deleter $deleter */
			$deleter = $this->service('xenMade\LD:LinkComment\Deleter', $linkComment);

			if ($this->filter('author_alert', 'bool') && $linkComment->canSendModeratorActionAlert())
			{
				$deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
			}

			$deleter->delete($type, $reason);

			$this->plugin('XF:InlineMod')->clearIdFromCookie('ld_link_comment', $linkComment->comment_id);

			return $this->redirect($this->getDynamicRedirect($this->buildLink('link-directory/comments', $linkComment->Link), false));
		}
		else
		{
			$viewParams = [
				'linkComment' => $linkComment,
				'link' => $linkComment->Link
			];

			return $this->view('xenMade\LD:LinkComment\Delete', 'ld_link_comment_delete', $viewParams);
		}
	}

    public function actionUndelete(ParameterBag $params)
    {
        $linkComment = $this->assertViewableLinkComment($params->comment_id);
        if (!$linkComment->canDelete('soft', $error))
        {
            return $this->noPermission($error);
        }

        /** @var \xenMade\LD\Service\LinkComment\Deleter $deleter */
        $deleter = $this->service('xenMade\LD:LinkComment\Deleter', $linkComment);

        if ($this->filter('author_alert', 'bool') && $linkComment->canSendModeratorActionAlert())
        {
            $deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
        }

        $deleter->undelete();

        return $this->redirect($this->getDynamicRedirect($this->buildLink('link-directory/comments', $linkComment->Link), false));
    }

	public function actionIp(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);

		/** @var \XF\ControllerPlugin\Ip $ipPlugin */
		$ipPlugin = $this->plugin('XF:Ip');
		return $ipPlugin->actionIp($linkComment);
	}

	public function actionReport(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);
		if (!$linkComment->canReport($error))
		{
			return $this->noPermission($error);
		}

		/** @var \XF\ControllerPlugin\Report $reportPlugin */
		$reportPlugin = $this->plugin('XF:Report');
		return $reportPlugin->actionReport(
			'ld_link_comment', $linkComment,
			$this->buildLink('link-directory/comments/report', $linkComment),
			$this->buildLink('link-directory/comments/view', $linkComment)
		);
	}

	public function actionLike(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);
		if (!$linkComment->canLike($error))
		{
			return $this->noPermission($error);
		}

		/** @var \XF\ControllerPlugin\Like $likePlugin */
		$likePlugin = $this->plugin('XF:Like');
		return $likePlugin->actionToggleLike(
			$linkComment,
			$this->buildLink('link-directory/comment/like', $linkComment),
			$this->buildLink('link-directory', $linkComment),
			$this->buildLink('link-directory/comment/likes', $linkComment)
		);
	}

	public function actionLikes(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);

		/** @var \XF\ControllerPlugin\Like $likePlugin */
		$likePlugin = $this->plugin('XF:Like');
		return $likePlugin->actionLikes(
			$linkComment,
			['link-directory/likes', $linkComment]
		);
	}

	public function actionWarn(ParameterBag $params)
	{
		$linkComment = $this->assertViewableLinkComment($params->comment_id);

		if (!$linkComment->canWarn($error))
		{
			return $this->noPermission($error);
		}

		/** @var \XF\ControllerPlugin\Warn $warnPlugin */
		$warnPlugin = $this->plugin('XF:Warn');
		return $warnPlugin->actionWarn('ld_link_comment', $linkComment, $this->buildLink('link-directory/comments/warn', $linkComment));
	}

	protected function setupEdit(\XF\Mvc\Entity\Entity $linkComment)
	{
		$message = $this->filter('message', 'str');

		/** @var \xenMade\LD\Service\LinkComment\Editor $editor */
		$editor = $this->service('xenMade\LD:LinkComment\Editor', $linkComment);
		$editor->setMessage($message);

		if ($this->filter('author_alert', 'bool') && $linkComment->canSendModeratorActionAlert())
		{
			$editor->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
		}

		return $editor;
	}

	protected function assertViewableLinkComment($linkCommentId, array $extraWith = [])
	{
		$extraWith[] = 'Link';
		array_unique($extraWith);

		/** @var \xenMade\LD\Entity\LinkComment $linkComment */
		$linkComment = $this->em()->find('xenMade\LD:LinkComment', $linkCommentId, $extraWith);

		if (!$linkComment)
		{
			throw $this->exception($this->notFound(\XF::phrase('ld_requested_link_comment_not_found')));
		}
		if (!$linkComment->canView($error))
		{
			throw $this->exception($this->noPermission($error));
		}

		return $linkComment;
	}


	/**
	 * @return \xenMade\LD\Repository\LinkComment
	 */
	protected function getLinkCommentRepo()
	{
		return $this->repository('xenMade\LD:LinkComment');
	}

	public static function getActivityDetails(array $activities)
	{
		return \XF::phrase('ld_viewing_link_directory');
	}
}
