<?php

namespace xenMade\LD\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class LinkRating extends AbstractController
{
	public function actionIndex(ParameterBag $params)
	{
        if(!$params->rating_id)
        {
            return $this->redirectPermanently($this->buildLink('link-directory'));
        }

        $linkRating = $this->assertViewableLinkRating($params->rating_id);

        if ($this->filter('_xfWithData', 'bool'))
        {
            $this->request->set('_xfDisableInlineMod', true);
            return $this->rerouteController(__CLASS__, 'show', $params);
        }

        $linkRatingRepo = $this->getLinkRatingRepo();

        $linkRatingFinder = $linkRatingRepo->findRatingsInLink($linkRating->Link);
        $linkRatings = $linkRatingFinder->where('rating_date', '>', $linkRating->rating_date)->fetch();

        $page = floor($linkRatings->count() / $this->options()->messagesPerPage) + 1;

        return $this->redirectPermanently($this->buildLink('link-directory/ratings', $linkRating->Link, ['page' => $page]) . '#link-comment-' . $linkRating->rating_id);
	}

	public function actionShow(ParameterBag $params)
	{
        $linkRating = $this->assertViewableLinkRating($params->rating_id);

        $viewParams = [
            'linkRating' => $linkRating
        ];

        return $this->view('xenMade\LD:LinkRating\Show', 'ld_link_rating', $viewParams);
	}

	public function actionDelete(ParameterBag $params)
	{
		$linkRating = $this->assertViewableLinkRating($params->rating_id);
		if (!$linkRating->canDelete('soft', $error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			$type = $this->filter('hard_delete', 'bool') ? 'hard' : 'soft';
			$reason = $this->filter('reason', 'str');

			if (!$linkRating->canDelete($type, $error))
			{
				return $this->noPermission($error);
			}

			/** @var \xenMade\LD\Service\LinkRating\Deleter $deleter */
			$deleter = $this->service('xenMade\LD:LinkRating\Deleter', $linkRating);

			if ($this->filter('author_alert', 'bool') && $linkRating->canSendModeratorActionAlert())
			{
				$deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
			}

			$deleter->delete($type, $reason);

			$this->plugin('XF:InlineMod')->clearIdFromCookie('ld_link_rating', $linkRating->rating_id);

			return $this->redirect($this->getDynamicRedirect($this->buildLink('link-directory', $linkRating->Link), false));
		}
		else
		{
			$viewParams = [
				'linkRating' => $linkRating,
				'link' => $linkRating->Link
			];

			return $this->view('xenMade\LD:LinkRating\Delete', 'ld_link_rating_delete', $viewParams);
		}
	}

    public function actionUndelete(ParameterBag $params)
    {
        $linkRating = $this->assertViewableLinkRating($params->rating_id);
        if (!$linkRating->canDelete('soft', $error))
        {
            return $this->noPermission($error);
        }

        /** @var \xenMade\LD\Service\LinkRating\Deleter $deleter */
        $deleter = $this->service('xenMade\LD:LinkRating\Deleter', $linkRating);

        if ($this->filter('author_alert', 'bool') && $linkRating->canSendModeratorActionAlert())
        {
            $deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
        }

        $deleter->undelete();

        return $this->redirect($this->getDynamicRedirect($this->buildLink('link-directory/ratings', $linkRating->Link), false));
    }

	public function actionIp(ParameterBag $params)
	{
		$linkRating = $this->assertViewableLinkRating($params->rating_id);

		/** @var \XF\ControllerPlugin\Ip $ipPlugin */
		$ipPlugin = $this->plugin('XF:Ip');
		return $ipPlugin->actionIp($linkRating);
	}

	public function actionReport(ParameterBag $params)
	{
		$linkRating = $this->assertViewableLinkRating($params->rating_id);
		if (!$linkRating->canReport($error))
		{
			return $this->noPermission($error);
		}

		/** @var \XF\ControllerPlugin\Report $reportPlugin */
		$reportPlugin = $this->plugin('XF:Report');
		return $reportPlugin->actionReport(
			'ld_link_rating', $linkRating,
			$this->buildLink('link-directory/ratings/report', $linkRating),
			$this->buildLink('link-directory/ratings/view', $linkRating)
		);
	}

	public function actionWarn(ParameterBag $params)
	{
		$linkRating = $this->assertViewableLinkRating($params->rating_id);

		if (!$linkRating->canWarn($error))
		{
			return $this->noPermission($error);
		}

		/** @var \XF\ControllerPlugin\Warn $warnPlugin */
		$warnPlugin = $this->plugin('XF:Warn');
		return $warnPlugin->actionWarn('ld_link_rating', $linkRating, $this->buildLink('link-directory/ratings/warn', $linkRating));
	}

	protected function setupEdit(\XF\Mvc\Entity\Entity $linkRating)
	{
		$message = $this->filter('message', 'str');

		/** @var \xenMade\LD\Service\LinkRating\Editor $editor */
		$editor = $this->service('xenMade\LD:LinkRating\Editor', $linkRating);
		$editor->setMessage($message);

		if ($this->filter('author_alert', 'bool') && $linkRating->canSendModeratorActionAlert())
		{
			$editor->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
		}

		return $editor;
	}

	protected function assertViewableLinkRating($ratingId, array $extraWith = [])
	{
		$extraWith[] = 'Link';
		array_unique($extraWith);

		/** @var \xenMade\LD\Entity\LinkRating $linkRating */
		$linkRating = $this->em()->find('xenMade\LD:LinkRating', $ratingId, $extraWith);

		if (!$linkRating)
		{
			throw $this->exception($this->notFound(\XF::phrase('ld_requested_link_rating_not_found')));
		}
		if (!$linkRating->canView($error))
		{
			throw $this->exception($this->noPermission($error));
		}

		return $linkRating;
	}


	/**
	 * @return \xenMade\LD\Repository\LinkRating
	 */
	protected function getLinkRatingRepo()
	{
		return $this->repository('xenMade\LD:LinkRating');
	}

	public static function getActivityDetails(array $activities)
	{
		return \XF::phrase('ld_viewing_link_directory');
	}
}
