<?php

namespace xenMade\LD\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class LinkUser extends AbstractController
{
	protected function preDispatchController($action, ParameterBag $params)
	{
		if (!\XF::visitor()->canViewLinks($error))
		{
			throw $this->exception($this->noPermission($error));
		}
	}

	public function actionIndex(ParameterBag $params)
	{
		if ($params->user_id)
		{
			return $this->rerouteController('xenMade\LD:LinkUser', 'User', $params);
		}

		/** @var \XF\Entity\MemberStat $memberStat */
		$memberStat = $this->em()->findOne('XF:MemberStat', ['member_stat_key' => 'ld_most_links']);

		if ($memberStat && $memberStat->canView())
		{
			return $this->redirectPermanently(
				$this->buildLink('members', null, ['key' => $memberStat->member_stat_key])
			);
		}
		else
		{
			return $this->redirect($this->buildLink('link-directory'));
		}
	}

	public function actionUser(ParameterBag $params)
	{
		/** @var \XF\Entity\User $user */
		$user = $this->assertRecordExists('XF:User', $params->user_id);

		$viewableCategoryIds = $this->repository('xenMade\LD:Category')->getViewableCategoryIds();

		/** @var \xenMade\LD\Repository\Link $linkRepo */
        $linkRepo = $this->repository('xenMade\LD:Link');
		$finder = $linkRepo->findLinksByUser($user->user_id, $viewableCategoryIds);

		$total = $finder->total();

		$page = $this->filterPage();
		$perPage = $this->options()->ldLinksPerPage;

		$this->assertValidPage($page, $perPage, $total, 'link-directory/user', $user);
		$this->assertCanonicalUrl($this->buildLink('link-directory/user', $user, ['page' => $page]));

		$links = $finder->limitByPage($page, $perPage)->fetch();
        $links = $links->filterViewable();

        $canInlineMod = false;
		foreach ($links AS $link)
		{
			/** @var \xenMade\LD\Entity\Link $link */
			if ($link->canUseInlineModeration())
			{
				$canInlineMod = true;
				break;
			}
		}

        $viewParams = [
			'user' => $user,
			'links' => $links,
			'page' => $page,
			'perPage' => $perPage,
			'total' => $total,
			'canInlineMod' => $canInlineMod
		];

		return $this->view('xenMade\LD:Author\View', 'ld_user_view', $viewParams);
	}

	public static function getActivityDetails(array $activities)
	{
		return \XF::phrase('ld_viewing_links');
	}
}