<?php

namespace xenMade\LD\Report;

use XF\Entity\Report;
use XF\Mvc\Entity\Entity;
use XF\Report\AbstractHandler;

class Link extends AbstractHandler
{
	protected function canViewContent(Report $report)
	{
		/** @var \xenMade\LD\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		$categoryId = $report->content_info['link']['category_id'];

		if (!method_exists($visitor, 'hasLinkCategoryPermission'))
		{
			return false;
		}

		return $visitor->hasLinkCategoryPermission($categoryId, 'view');
	}

	protected function canActionContent(Report $report)
	{
		/** @var \xenMade\LD\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		$categoryId = $report->content_info['link']['category_id'];

		if (!method_exists($visitor, 'hasLinkCategoryPermission'))
		{
			return false;
		}

		return (
			$visitor->hasLinkCategoryPermission($categoryId, 'deleteAny')
			|| $visitor->hasLinkCategoryPermission($categoryId, 'editAny')
		);
	}

	public function setupReportEntityContent(Report $report, Entity $content)
	{
		/** @var \xenMade\LD\Entity\Link $link */
		$category = $content->Category;

		if (!empty($content->prefix_id))
		{
			$title = $content->Prefix->title . ' - ' . $content->title;
		}
		else
		{
			$title = $content->title;
		}

		$report->content_user_id = $content->user_id;
		$report->content_info = [
			'link' => [
				'link_id' => $content->link_id,
				'title' => $title,
				'prefix_id' => $content->prefix_id,
				'category_id' => $content->category_id,
				'user_id' => $content->user_id,
				'username' => $content->username,
				'description' => $content->description,
			],
			'category' => [
				'category_id' => $category->category_id,
				'title' => $category->title
			]
		];
	}

	public function getContentTitle(Report $report)
	{
		return \XF::phrase('ld_link_in_x', [
			'title' => $report->content_info['link']['title']
		]);
	}

	public function getContentMessage(Report $report)
	{
		return $report->content_info['link']['description'];
	}

	public function getContentLink(Report $report)
	{
		$info = $report->content_info;

		return \XF::app()->router()->buildLink(
			'canonical:link-directory',
			[
				'link_id' => $info['link']['link_id'],
				'title' => $info['link']['title'],
			]
		);
	}

	public function getEntityWith()
	{
		return ['Category'];
	}
}