<?php

namespace xenMade\LD\Report;

use XF\Entity\Report;
use XF\Mvc\Entity\Entity;
use XF\Report\AbstractHandler;

class LinkComment extends AbstractHandler
{
	protected function canViewContent(Report $report)
	{
		return \XF::visitor()->canViewLinkComments();
	}

	protected function canActionContent(Report $report)
	{
		$visitor = \XF::visitor();
		return ($visitor->hasPermission('ld_link_comment', 'editAny') || $visitor->hasPermission('ld_link_comment', 'deleteAny'));
	}

	public function setupReportEntityContent(Report $report, Entity $content)
	{
        $comment = $content;
        $link = $content->Link;
        $category = $link->Category;

        if (!empty($link->prefix_id))
        {
            $title = $link->Prefix->title . ' - ' . $link->title;
        }
        else
        {
            $title = $link->title;
        }

		$report->content_user_id = $content->user_id;
		$report->content_info = [
            'comment' => [
                'comment_id' => $comment->comment_id,
                'link_id' => $comment->link_id,
                'message' => $comment->message,
                'user_id'
            ],
            'link' => [
                'link_id' => $link->link_id,
                'title' => $title,
                'prefix_id' => $link->prefix_id,
                'category_id' => $link->category_id,
                'user_id' => $link->user_id,
                'username' => $link->username
            ],
            'category' => [
                'category_id' => $category->category_id,
                'title' => $category->title
            ]
		];
	}

	public function getContentTitle(Report $report)
	{
        return \XF::phrase('ld_link_comment_in_x', [
            'title' => \XF::app()->stringFormatter()->censorText($report->content_info['link']['title'])
        ]);
	}

	public function getContentMessage(Report $report)
	{
		return $report->content_info['message'];
	}

	public function getContentLink(Report $report)
	{
        $info = $report->content_info;

        return \XF::app()->router()->buildLink(
            'canonical:link-directory/comments',
            [
                'link_id' => $info['link']['link_id'],
                'title' => $info['link']['title'],
                'comment_id' => $info['comment']['comment_id']
            ]
        );
	}

	public function getEntityWith()
	{
		return ['Link'];
	}
}