<?php

namespace xenMade\LD\Report;

use XF\Entity\Report;
use XF\Mvc\Entity\Entity;
use XF\Report\AbstractHandler;

class LinkRating extends AbstractHandler
{
	protected function canViewContent(Report $report)
	{
		$visitor = \XF::visitor();
		$categoryId = $report->content_info['link']['category_id'];

		if (!method_exists($visitor, 'hasLinkCategoryPermission'))
		{
			return false;
		}

		return $visitor->hasLinkCategoryPermission($categoryId, 'view');
	}

	protected function canActionContent(Report $report)
	{
		$visitor = \XF::visitor();
		$categoryId = $report->content_info['link']['category_id'];

		if (!method_exists($visitor, 'hasLinkCategoryPermission'))
		{
			return false;
		}

		return (
			$visitor->hasLinkCategoryPermission($categoryId, 'deleteAny')
		);
	}

	public function setupReportEntityContent(Report $report, Entity $content)
	{
		/** @var \xenMade\LD\Entity\LinkRating $rating */
		$rating = $content;
		$link = $content->Link;
		$category = $link->Category;

		if (!empty($link->prefix_id))
		{
			$title = $link->Prefix->title . ' - ' . $link->title;
		}
		else
		{
			$title = $link->title;
		}

		$report->content_user_id = $rating->user_id;
		$report->content_info = [
			'rating' => [
				'rating_id' => $rating->rating_id,
				'link_id' => $rating->link_id,
				'rating' => $rating->rating,
				'message' => $rating->message
			],
			'link' => [
				'link_id' => $link->link_id,
				'title' => $title,
				'prefix_id' => $link->prefix_id,
				'category_id' => $link->category_id,
				'user_id' => $link->user_id,
				'username' => $link->username
			],
			'category' => [
				'category_id' => $category->category_id,
				'title' => $category->title
			]
		];
	}

	public function getContentTitle(Report $report)
	{
		return \XF::phrase('ld_link_rating_in_x', [
			'title' => \XF::app()->stringFormatter()->censorText($report->content_info['link']['title'])
		]);
	}

	public function getContentMessage(Report $report)
	{
		return $report->content_info['rating']['message'];
	}

	public function getContentLink(Report $report)
	{
		$info = $report->content_info;

		return \XF::app()->router()->buildLink(
			'canonical:link-directory/ratings',
			[
				'link_id' => $info['link']['link_id'],
				'title' => $info['link']['title'],
				'rating_id' => $info['rating']['rating_id']
			]
		);
	}

	public function getEntityWith()
	{
		return ['Link', 'Link.Category'];
	}
}