<?php

namespace xenMade\LD\Repository;

use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;

abstract class AbstractUpgradeMap extends Repository
{
    abstract protected function getMapEntityIdentifier();

    abstract protected function getAssociationsForUpgrade(\xenMade\LD\Entity\LinkUpgrade $upgrade);

    abstract protected function updateAssociationCache(array $cache);

    protected function getStructureData()
    {
        $mapEntity = $this->getMapEntityIdentifier();
        $structure = $this->em->getEntityStructure($mapEntity);
        $className = $this->em->getEntityClassName($mapEntity);

        return [
            'mapEntity' => $mapEntity,
            'table' => $structure->table,
            'key' => $className::getContainerKey()
        ];
    }

    public function updateUpgradeAssociations(\xenMade\LD\Entity\LinkUpgrade $upgrade, array $contentIds)
    {
        $emptyKey = array_search(0, $contentIds);
        if ($emptyKey !== false)
        {
            unset($contentIds[$emptyKey]);
        }
        $contentIds = array_unique($contentIds);

        $structureData = $this->getStructureData();

        $existingAssociations = $this->getAssociationsForUpgrade($upgrade);
        if (!count($existingAssociations) && !$contentIds)
        {
            return;
        }

        $db = $this->db();
        $db->beginTransaction();

        $db->delete($structureData['table'], 'upgrade_id = ?', $upgrade->upgrade_id);

        $map = [];

        foreach ($contentIds AS $id)
        {
            $map[] = [
                $structureData['key'] => $id,
                'upgrade_id' => $upgrade->upgrade_id
            ];
        }
        if ($map)
        {
            $db->insertBulk($structureData['table'], $map);
        }

        $rebuildIds = $contentIds;

        foreach ($existingAssociations AS $association)
        {
            /** @var \XF\Mvc\Entity\Entity $association */
            $rebuildIds[] = $association->getValue($structureData['key']);
        }

        $rebuildIds = array_unique($rebuildIds);
        $this->rebuildContentAssociationCache($rebuildIds);

        $db->commit();
    }

    public function removeUpgradeAssociations(\xenMade\LD\Entity\LinkUpgrade $upgrade)
    {
        $structureData = $this->getStructureData();

        $rebuildIds = $this->db()->fetchAllColumn("
			SELECT $structureData[key]
			FROM $structureData[table]
			WHERE upgrade_id = ?
		", $upgrade->upgrade_id);

        if (!$rebuildIds)
        {
            return;
        }

        $db = $this->db();
        $db->beginTransaction();

        $db->delete($structureData['table'], 'upgrade_id = ?', $upgrade->upgrade_id);

        $this->rebuildContentAssociationCache($rebuildIds);

        $db->commit();
    }

    public function updateContentAssociations($contentId, array $upgradeIds)
    {
        $structureData = $this->getStructureData();

        $db = $this->db();
        $db->beginTransaction();

        $db->delete($structureData['table'], $structureData['key'] . ' = ?', $contentId);

        $map = [];

        foreach ($upgradeIds AS $upgradeId)
        {
            $map[] = [
                $structureData['key'] => $contentId,
                'upgrade_id' => $upgradeId
            ];
        }

        if ($map)
        {
            $db->insertBulk($structureData['table'], $map);
        }

        $this->rebuildContentAssociationCache($contentId);

        $db->commit();
    }

    public function rebuildContentAssociationCache($contentIds)
    {
        if (!is_array($contentIds))
        {
            $contentIds = [$contentIds];
        }
        if (!$contentIds)
        {
            return;
        }

        $structureData = $this->getStructureData();

        $newCache = [];

        $upgradeAssociations = $this->finder($structureData['mapEntity'])
            ->with('Upgrade')
            ->where($structureData['key'], $contentIds)
            ->order('Upgrade.display_order');

        foreach ($upgradeAssociations->fetch() AS $upgradeMap)
        {
            $key = $upgradeMap->get($structureData['key']);
            $newCache[$key][$upgradeMap->upgrade_id] = $upgradeMap->upgrade_id;
        }

        foreach ($contentIds AS $contentId)
        {
            if (!isset($newCache[$contentId]))
            {
                $newCache[$contentId] = [];
            }
        }

        $this->updateAssociationCache($newCache);
    }
}