<?php

namespace xenMade\LD\Repository;

use XF\Mvc\Entity\Repository;

class Link extends Repository
{
	public function findLinksForLinkList(array $viewableCategoryIds = null, array $limits = [])
	{
		$limits = array_replace([
			'visibility' => true,
			'allowOwnPending' => false
		], $limits);

		/** @var \xenMade\LD\Finder\Link $linkFinder */
		$linkFinder = $this->finder('xenMade\LD:Link');

		if (is_array($viewableCategoryIds))
		{
			$linkFinder->where('category_id', $viewableCategoryIds);
		}
		else
		{
			$linkFinder->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);
		}

		$linkFinder
			->forFullView(true)
			->useDefaultOrder();

		if ($limits['visibility'])
		{
			$linkFinder->applyGlobalVisibilityChecks($limits['allowOwnPending']);
		}

		return $linkFinder;
	}

	public function findTOPLinks(array $viewableCategoryIds = null)
	{
		/** @var \xenMade\LD\Finder\Link $linkFinder */
		$linkFinder = $this->finder('xenMade\LD:Link');

		if (is_array($viewableCategoryIds))
		{
			$linkFinder->where('category_id', $viewableCategoryIds);
		}
		else
		{
			$linkFinder->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);
		}

		$linkFinder
			->where('state', 'visible')
			->where('rating_count', '>', 0)
			->with(['User'])
			->setDefaultOrder('rating_weighted', 'desc');

		return $linkFinder;
	}

	public function findNewLinks(array $viewableCategoryIds = null)
	{
		/** @var \xenMade\LD\Finder\Link $linkFinder */
		$linkFinder = $this->finder('xenMade\LD:Link');

		if (is_array($viewableCategoryIds))
		{
			$linkFinder->where('category_id', $viewableCategoryIds);
		}
		else
		{
			$linkFinder->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);
		}

		$linkFinder
			->where('state', 'visible')
            ->with(['User'])
			->order('insert_date', 'DESC');

		return $linkFinder;
	}

    public function findVIPLinks(array $viewableCategoryIds = null)
    {
        /** @var \xenMade\LD\Finder\Link $linkFinder */
        $linkFinder = $this->finder('xenMade\LD:Link');

        if (is_array($viewableCategoryIds))
        {
            $linkFinder->where('category_id', $viewableCategoryIds);
        }
        else
        {
            $linkFinder->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);
        }

        $linkFinder
            ->where('state', 'visible')
            ->where('is_vip', '=', 1)
            ->with(['User'])
            ->setDefaultOrder('insert_date', 'desc');

        return $linkFinder;
    }

    public function findRandomLinks(array $viewableCategoryIds = null)
    {
        /** @var \xenMade\LD\Finder\Link $linkFinder */
        $linkFinder = $this->finder('xenMade\LD:Link');

        if (is_array($viewableCategoryIds))
        {
            $linkFinder->where('category_id', $viewableCategoryIds);
        }
        else
        {
            $linkFinder->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);
        }

        $linkFinder
            ->where('state', 'visible')
            ->where('is_vip', '=', 0)
            ->with(['User'])
            ->order(\XF\Mvc\Entity\Finder::ORDER_RANDOM);

        return $linkFinder;
    }

	public function findLinksForWatchedList($userId = null)
	{
		if ($userId === null)
		{
			$userId = \XF::visitor()->user_id;
		}
		$userId = intval($userId);

		/** @var \xenMade\LD\Finder\Link $finder */
		$finder = $this->finder('xenMade\LD:Link');

		$finder
			->forFullView(true)
			->with('Watch|' . $userId, true)
			->where('state', 'visible')
			->setDefaultOrder('last_update', 'DESC');

		return $finder;
	}

	public function findLinksByUser($userId, array $viewableCategoryIds = null, array $limits = [])
	{
		/** @var \xenMade\LD\Finder\Link $linkFinder */
		$linkFinder = $this->finder('xenMade\LD:Link');

		$linkFinder->where('user_id', $userId)
			->forFullView(true)
			->setDefaultOrder('last_update', 'desc');

		if (is_array($viewableCategoryIds))
		{
			$linkFinder->where('category_id', $viewableCategoryIds);
		}
		else
		{
			$linkFinder->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);
		}

		$limits = array_replace([
			'visibility' => true,
			'allowOwnPending' => $userId == \XF::visitor()->user_id
		], $limits);

		if ($limits['visibility'])
		{
			$linkFinder->applyGlobalVisibilityChecks($limits['allowOwnPending']);
		}

		return $linkFinder;
	}

	public function sendModeratorActionAlert(\xenMade\LD\Entity\Link $link, $action, $reason = '', array $extra = [], \XF\Entity\User $forceUser = null)
	{
		if (!$forceUser)
		{
			if (!$link->user_id || !$link->User)
			{
				return false;
			}

			$forceUser = $link->User;
		}

		$extra = array_merge([
			'title' => $link->title,
			'prefix_id' => $link->prefix_id,
			'link' => $this->app()->router('public')->buildLink('nopath:link-directory', $link),
			'reason' => $reason
		], $extra);

		/** @var \XF\Repository\UserAlert $alertRepo */
		$alertRepo = $this->repository('XF:UserAlert');
		$alertRepo->alert(
			$forceUser,
			0,
            '',
			'ld_link',
            $forceUser->user_id,
			$action,
            $extra
		);

		return true;
	}

	public function getUserCount($userId)
	{
		return $this->db()->fetchOne("
			SELECT COUNT(*)
			FROM xf_xenmade_ld_link
			WHERE user_id = ?
				AND state = 'visible'
		", $userId);
	}

    public function findLinkForThread(\XF\Entity\Thread $thread)
    {
        /** @var \xenMade\LD\Finder\Link $finder */
        $finder = $this->finder('xenMade\LD:Link');

        $finder->where('thread_id', $thread->thread_id)
            ->forFullView()
            ->with('Category.Permissions|' . \XF::visitor()->permission_combination_id);

        return $finder;
    }

}