<?php

namespace xenMade\LD\Repository;

use XF\Mvc\Entity\Repository;

class LinkComment extends Repository
{
	public function findCommentsOnLink(\xenMade\LD\Entity\Link $link, array $limits = [])
	{
		/** @var \xenMade\LD\Finder\LinkComment $finder */
		$finder = $this->finder('xenMade\LD:LinkComment');
		$finder
			->onLink($link, $limits)
			->order('post_date', 'DESC');

		return $finder;
	}

	public function findNewestLinkCommentsOnLink(\xenMade\LD\Entity\Link $link, $newerThan, array $limits = [])
	{
		/** @var \xenMade\LD\Finder\LinkComment $finder */
		$finder = $this->findNewestLinkComments($newerThan)
			->onLink($link, $limits);

		return $finder;
	}

	public function findNewestLinkComments($newerThan)
	{
		/** @var \xenMade\LD\Finder\LinkComment $finder */
		$finder = $this->finder('xenMade\LD:LinkComment');
		$finder
			->newerThan($newerThan)
			->order('post_date', 'DESC');

		return $finder;
	}

    public function findLatestLinkCommentOnLink(\xenMade\LD\Entity\Link $link)
    {
        /** @var \xenMade\LD\Finder\LinkComment $finder */
        $finder = $this->finder('xenMade\LD:LinkComment');
        $finder
            ->forFullView()
            ->where('link_id', $link->link_id)
            ->where('message_state', 'visible')
            ->order('post_date', 'DESC');

        return $finder;
    }

	public function sendModeratorActionAlert(\xenMade\LD\Entity\LinkComment $linkComment, $action, $reason = '', array $extra = [])
	{
		if (!$linkComment->user_id || !$linkComment->Link)
		{
			return false;
		}

		$router = $this->app()->router('public');

		$extra = array_merge([
			'linkId' => $linkComment->link_id,
			'linkCommentUser' => $linkComment ? $linkComment->username : '',
			'linkCommentLink' => $router->buildLink('nopath:link-directory', $linkComment->Link),
			'link' => $router->buildLink('nopath:link-directory/comments', $linkComment->Link),
			'reason' => $reason
		], $extra);

		/** @var \XF\Repository\UserAlert $alertRepo */
		$alertRepo = $this->repository('XF:UserAlert');
		$alertRepo->alert(
            $linkComment->User,
			0, '',
			'ld_link_comment', $linkComment->comment_id,
			$action, $extra
		);

		return true;
	}
}