<?php

namespace xenMade\LD\Repository;

use XF\Mvc\Entity\Repository;

class LinkRating extends Repository
{
	public function findRatingsInLink(\xenMade\LD\Entity\Link $link, array $limits = [])
	{
		/** @var \xenMade\LD\Finder\LinkRating $finder */
		$finder = $this->finder('xenMade\LD:LinkRating');
		$finder->inLink($link, $limits)
			->setDefaultOrder('rating_date', 'desc');

		return $finder;
	}

	public function findLatestRatings(array $viewableCategoryIds = null)
	{
		/** @var \xenMade\LD\Finder\LinkRating $finder */
		$finder = $this->finder('xenMade\LD:LinkRating');

		if (is_array($viewableCategoryIds))
		{
			$finder->where('Link.category_id', $viewableCategoryIds);
		}
		else
		{
			$finder->with('Link.Category.Permissions|' . \XF::visitor()->permission_combination_id);
		}

		$finder->where([
				'Link.state' => 'visible',
				'rating_state' => 'visible'
			])
			->with('Link', true)
			->with(['Link.Category', 'User'])
			->setDefaultOrder('rating_date', 'desc');

		$cutOffDate = \XF::$time - ($this->options()->readMarkingDataLifetime * 86400);
		$finder->where('rating_date', '>', $cutOffDate);

		return $finder;
	}

    public function findLatestRatingOnLink(\xenMade\LD\Entity\Link $link)
    {
        /** @var \xenMade\LD\Finder\LinkRating $finder */
        $finder = $this->finder('xenMade\LD:LinkRating');
        $finder
            ->forFullView()
            ->with('Link', true)
            ->where('link_id', $link->link_id)
            ->where('rating_state', 'visible')
            ->order('rating_date', 'DESC');

        return $finder;
    }

	public function getCountableRating($linkId, $userId)
	{
		/** @var \xenMade\LD\Finder\LinkRating $finder */
		$finder = $this->finder('xenMade\LD:LinkRating');
		$finder->where([
			'link_id' => $linkId,
			'user_id' => $userId,
			'rating_state' => 'visible'
		])->order('rating_date', 'desc');

		return $finder->fetchOne();
	}

	public function getCountedRatings($linkId, $userId)
	{
		/** @var \xenMade\LD\Finder\LinkRating $finder */
		$finder = $this->finder('xenMade\LD:LinkRating');
		$finder->where([
			'link_id' => $linkId,
			'user_id' => $userId
		])->order('rating_date', 'desc');

		return $finder->fetch();
	}

	public function sendModeratorActionAlert(\xenMade\LD\Entity\LinkRating $rating, $action, $reason = '', array $extra = [])
	{
		$link = $rating->Link;

		if (!$link || !$link->user_id || !$link->User)
		{
			return false;
		}

		$extra = array_merge([
			'title' => $link->title,
			'prefix_id' => $link->prefix_id,
			'rating' => $this->app()->router('public')->buildLink('nopath:link-directory/ratings', $rating),
			'link' => $this->app()->router('public')->buildLink('nopath:link-directory', $link),
			'reason' => $reason
		], $extra);

		/** @var \XF\Repository\UserAlert $alertRepo */
		$alertRepo = $this->repository('XF:UserAlert');
		$alertRepo->alert(
			$rating->User,
			0, '',
			'user', $rating->user_id,
			"ld_link_rating_{$action}", $extra
		);

		return true;
	}

	public function sendRatingAlertToLinkAuthor(\xenMade\LD\Entity\LinkRating $rating)
	{
		if (!$rating->isVisible())
		{
			return false;
		}

		$link = $rating->Link;
		$linkAuthor = $link->User;

		if (!$linkAuthor)
		{
			return false;
		}

        $senderId = $rating->user_id;
        $senderName = $rating->User ? $rating->User->username : \XF::phrase('unknown')->render('raw');

		$alertRepo = $this->repository('XF:UserAlert');
		return $alertRepo->alert(
			$linkAuthor, $senderId, $senderName, 'ld_link_rating', $rating->rating_id, 'rate'
		);
	}
}