<?php

namespace xenMade\LD\Repository;

use XF\Mvc\Entity\Repository;

class LinkUpgrade extends Repository
{
	public function findUpgradeForList()
	{
        $finder = $this->finder('xenMade\LD:LinkUpgrade')
            ->order('display_order');

        return $finder;
	}

    public function getFilteredUpgradesForList(\xenMade\LD\Entity\Link $link)
    {
        $finder = $this->findUpgradeForList()
            ->with('Active|' . $link->link_id)
            ->where('Category.category_id', '=', $link->Category->category_id);

        $purchased = null;
        $upgrades = $finder->fetch();

        if ($upgrades->count())
        {
            foreach ($upgrades AS $upgradeId => $upgrade)
            {
                if (!$upgrade->can_purchase)
                {
                    unset($upgrades[$upgradeId]);
                    continue;
                }

                if (isset($link->link_id) && isset($upgrade->Active[$link->link_id]))
                {
                    if(empty($purchased))
                    {
                        $purchased = $upgrade->Active[$link->link_id];
                    }

                    unset($upgrades[$upgradeId]);
                }
            }
        }

        return [$upgrades, $purchased];
    }

    public function expiredUpgrades()
    {
        /** @var \xenMade\LD\Entity\LinkUpgradeActive[] $expired */
        $expired = $this->finder('xenMade\LD:LinkUpgradeActive')
            ->with('Upgrade')
            ->with('Link')
            ->where('end_date', '<', \XF::$time)
            ->where('end_date', '>', 0)
            ->order('end_date')
            ->fetch(1000);

        foreach ($expired AS $active)
        {
            $upgrade = $active->Upgrade;

            if ($upgrade && $upgrade->recurring)
            {
                if ($active->end_date + 86400 >= \XF::$time)
                {
                    continue;
                }
            }

            if ($upgrade)
            {
                /** @var \xenMade\LD\Service\Link\Downgrade $downgradeService */
                $downgradeService = $this->app()->service('xenMade\LD:Link\Downgrade', $active->Upgrade, $active->Link, $active);
                $downgradeService->downgrade();
            }
        }
    }

    public function findActiveLinkUpgradesForList()
    {
        $finder = $this->finder('xenMade\LD:LinkUpgradeActive');

        $finder
            ->with(['Upgrade', 'Link', 'PurchaseRequest.PaymentProfile'])
            ->setDefaultOrder('start_date', 'DESC');

        return $finder;
    }

    public function findExpiredLinkUpgradesForList()
    {
        $finder = $this->finder('xenMade\LD:LinkUpgradeExpired');

        $finder
            ->with(['Upgrade', 'Link', 'PurchaseRequest.PaymentProfile'])
            ->setDefaultOrder('end_date', 'DESC');

        return $finder;
    }

    public function expireActiveUpgrade(\xenMade\LD\Entity\LinkUpgradeActive $active, \xenMade\LD\Entity\LinkUpgradeExpired $expired = null)
    {
        if ($expired === null)
        {
            $expired = $this->em->create('xenMade\LD:LinkUpgradeExpired');
        }

        $expired->upgrade_record_id = $active->upgrade_record_id;
        $expired->link_id = $active->link_id;
        $expired->purchase_request_key = $active->purchase_request_key;
        $expired->upgrade_id = $active->upgrade_id;
        $expired->extra = $active->extra;
        $expired->start_date = $active->start_date;
        $expired->end_date = time();
        $expired->original_end_date = $active->end_date;

        $expired->save(false, false);
        $active->delete(true, false);
    }
}