<?php

namespace xenMade\LD\Search\Data;

use XF\Mvc\Entity\Entity;
use XF\Search\Data\AbstractData;
use XF\Search\IndexRecord;
use XF\Search\MetadataStructure;

class LinkComment extends AbstractData
{
	public function getEntityWith($forView = false)
	{
		$get = ['Link'];
		if ($forView)
		{
			$get[] = 'User';
		}

		return $get;
	}

	public function getIndexData(Entity $entity)
	{
		/** @var \xenMade\LD\Entity\LinkComment $entity */
		if (!$entity->Link)
		{
			return null;
		}

		$index = IndexRecord::create('ld_link_comment', $entity->comment_id, [
			'title' => '',
			'message' => $entity->message_,
			'date' => $entity->post_date,
			'user_id' => $entity->user_id,
			'discussion_id' => $entity->comment_id,
			'metadata' => $this->getMetaData($entity)
		]);

		if (!$entity->isVisible())
		{
			$index->setHidden();
		}

		return $index;
	}

	protected function getMetaData(\xenMade\LD\Entity\LinkComment $entity)
	{
		$metadata = [];

		$metadata['link'] = $entity->link_id;

		return $metadata;
	}

	public function setupMetadataStructure(MetadataStructure $structure)
	{
		$structure->addField('link', MetadataStructure::INT);
	}

	public function getResultDate(Entity $entity)
	{
		return $entity->post_date;
	}

	public function getSearchableContentTypes()
	{
		return ['ld_link_comment'];
	}

	public function getTemplateData(Entity $entity, array $options = [])
	{
		return [
			'linkComment' => $entity,
			'options' => $options
		];
	}

	public function getSearchFormTab()
	{
		$visitor = \XF::visitor();
		if (!$visitor->canViewLinkComments())
		{
			return null;
		}

		return [
			'title' => \XF::phrase('ld_search_link_comments'),
			'order' => 1000
		];
	}

	public function getSectionContext()
	{
		return 'ld';
	}

	public function applyTypeConstraintsFromInput(\XF\Search\Query\Query $query, \XF\Http\Request $request, array &$urlConstraints)
	{
		$link = $request->filter('c.link', 'str');
		if ($link)
		{
			$users = preg_split('/,\s*/', $link, -1, PREG_SPLIT_NO_EMPTY);
			if ($users)
			{
				/** @var \XF\Repository\User $userRepo */
				$userRepo = \XF::repository('XF:User');
				$matchedUsers = $userRepo->getUsersByNames($users, $notFound);
				if ($notFound)
				{
					$query->error('users',
						\XF::phrase('following_members_not_found_x', ['members' => implode(', ', $notFound)])
					);
				}
				else
				{
					$query->withMetadata('link', $matchedUsers->keys());
					$urlConstraints['link'] = implode(', ', $users);
				}
			}
		}
	}

	public function canUseInlineModeration(Entity $entity, &$error = null)
	{
		/** @var \xenMade\LD\Entity\LinkComment $entity */
		return $entity->canUseInlineModeration($error);
	}
}