<?php

namespace xenMade\LD\Service\Link;

class Delete extends \XF\Service\AbstractService
{
	protected $link;

	protected $user;

	protected $alert = false;
	protected $alertReason = '';

	public function __construct(\XF\App $app, \xenMade\LD\Entity\Link $link)
	{
		parent::__construct($app);
		$this->link = $link;
	}

	public function getLink()
	{
		return $this->link;
	}

	public function setUser(\XF\Entity\User $user = null)
	{
		$this->user = $user;
	}

	public function getUser()
	{
		return $this->user;
	}

	public function setSendAlert($alert, $reason = null)
	{
		$this->alert = (bool)$alert;
		if ($reason !== null)
		{
			$this->alertReason = $reason;
		}
	}

	public function delete($type, $reason = '')
	{
		$user = $this->user ?: \XF::visitor();
		$wasVisible = $this->link->isVisible();

		if ($type == 'soft')
		{
			$result = $this->link->softDelete($reason, $user);
		}
		else
		{
			$result = $this->link->delete();
		}

		if ($result && $wasVisible && $this->alert && $this->link->user_id != $user->user_id)
		{
			/** @var \xenMade\LD\Repository\Link $linkRepo */
            $linkRepo = $this->repository('xenMade\LD:Link');
            $linkRepo->sendModeratorActionAlert($this->link, 'delete', $this->alertReason);
		}

        $this->updateThread();

		return $result;
	}


    protected function updateThread()
    {
        $link = $this->link;
        $thread = $link->Thread;

        if (!$thread)
        {
            return;
        }

        $asUser = $link->User ?: $this->repository('XF:User')->getGuestUser($link->username);

        \XF::asVisitor($asUser, function() use ($thread)
        {
            $replier = $this->setupThreadReply($thread);
            if ($replier && $replier->validate())
            {
                $existingLastPostDate = $replier->getThread()->last_post_date;

                $post = $replier->save();

                if (\XF::visitor()->user_id && $post->Thread->getVisitorReadDate() >= $existingLastPostDate)
                {
                    $this->repository('XF:Thread')->markThreadReadByVisitor($thread);
                }

                \XF::runLater(function() use ($replier)
                {
                    $replier->sendNotifications();
                });
            }
        });
    }

    protected function setupThreadReply(\XF\Entity\Thread $thread)
    {
        if (!$thread->Forum)
        {
            return null;
        }

        $link = $this->link;

        $deleteMessage = \XF::phrase('ld_link_thread_message_delete', [
            'title' => $link->title_,
            'username' => $link->User ? $link->User->username : $link->username
        ])
            ->render('raw');

        /** @var \XF\Service\Thread\Replier $replier */
        $replier = $this->service('XF:Thread\Replier', $thread);
        $replier->setIsAutomated();
        $replier->setMessage($deleteMessage, false);

        return $replier;
    }
}