<?php

namespace xenMade\LD\Service\Link;

class Downgrade extends \XF\Service\AbstractService
{
	protected $link;

	protected $upgrade;

	protected $activeUpgrade;

	protected $expiredUpgrade;

	protected $sendAlert = true;

	public function __construct(\XF\App $app, \xenMade\LD\Entity\LinkUpgrade $upgrade, \xenMade\LD\Entity\Link $link, \xenMade\LD\Entity\LinkUpgradeActive $active = null)
	{
		parent::__construct($app);

		$this->link = $link;
		$this->activeUpgrade = $active;
		$this->setUpgrade($upgrade);
	}

	public function getLink()
	{
		return $this->link;
	}

	protected function setUpgrade(\xenMade\LD\Entity\LinkUpgrade $upgrade)
	{
		$this->upgrade = $upgrade;
		$link = $this->link;

		if (!$this->activeUpgrade)
		{
			$activeUpgrades = $upgrade->Active;
			$this->activeUpgrade = isset($activeUpgrades[$link->link_id]) ? $activeUpgrades[$link->link_id] : null;
		}

		$this->expiredUpgrade = $this->em()->create('xenMade\LD:LinkUpgradeExpired');
	}

	public function getUpgrade()
	{
		return $this->upgrade;
	}

	public function getActiveUpgrade()
	{
		return $this->activeUpgrade;
	}

	public function getExpiredUpgrade()
	{
		return $this->expiredUpgrade;
	}

	public function setSendAlert($sendAlert)
	{
		$this->sendAlert = $sendAlert;
	}

	public function downgrade()
	{
		$link = $this->link;
		$upgrade = $this->upgrade;
		$active = $this->activeUpgrade;
		$expired = $this->expiredUpgrade;

		$db = $this->db();
		$db->beginTransaction();

		if ($active)
		{
			/** @var \xenMade\LD\Repository\LinkUpgrade $upgradeRepo */
			$upgradeRepo = $this->repository('xenMade\LD:LinkUpgrade');
			$upgradeRepo->expireActiveUpgrade($active, $expired);

            $link->set('is_vip', false);
            $link->save(false);

			if (!$upgrade->recurring && $upgrade->can_purchase && $this->sendAlert)
			{
				/** @var \XF\Repository\UserAlert $alertRepo */
				//$alertRepo = $this->app->repository('XF:UserAlert');
				//$alertRepo->alert($user, $user->user_id, $user->username, 'user', $user->user_id, 'upgrade_end');
			}
		}

		$db->commit();

		return true;
	}
}