<?php

namespace xenMade\LD\Service\Link;

class Edit extends \XF\Service\AbstractService
{
	use \XF\Service\ValidateAndSavableTrait;

	protected $link;
	protected $performValidations = true;
	protected $alert = false;
	protected $alertReason = '';

    /**
     * @var \xenMade\LD\Service\Link\Preparer
     */
    protected $preparer;

	public function __construct(\XF\App $app, \xenMade\LD\Entity\Link $link)
	{
		parent::__construct($app);
		$this->link = $link;

        $this->preparer = $this->service('xenMade\LD:Link\Preparer', $this->link);
	}

	public function getLink()
	{
		return $this->link;
	}

	public function setPerformValidations($perform)
	{
		$this->performValidations = (bool)$perform;
	}

	public function getPerformValidations()
	{
		return $this->performValidations;
	}

	public function setTitle($title)
	{
		$this->link->title = $title;
	}

	public function setDescription($description)
	{
		$this->link->description = $description;
	}

	public function setUrl($url)
	{
        return $this->preparer->setUrl($url);
	}

    public function setBacklink()
    {
        $this->preparer->setBacklink();
    }

	public function setPrefix($prefixId)
	{
		$this->link->prefix_id = $prefixId;
	}

	public function setCustomFields(array $customFields)
	{
		$link = $this->link;

		$editMode = $link->getFieldEditMode();

		/** @var \XF\CustomField\Set $fieldSet */
		$fieldSet = $link->custom_fields;
		$fieldDefinition = $fieldSet->getDefinitionSet()
			->filterEditable($fieldSet, $editMode)
			->filterOnly($link->Category->field_cache);

		$customFieldsShown = array_keys($fieldDefinition->getFieldDefinitions());

		if ($customFieldsShown)
		{
			$fieldSet->bulkSet($customFields, $customFieldsShown, $editMode);
		}
	}

	public function setSendAlert($alert, $reason = null)
	{
		$this->alert = (bool)$alert;
		if ($reason !== null)
		{
			$this->alertReason = $reason;
		}
	}

	public function checkForSpam()
	{
        if ($this->link->state == 'visible' && \XF::visitor()->isSpamCheckRequired())
        {
            /** @var \xenMade\LD\Entity\Link $link */
            $link = $this->link;

            /** @var \XF\Entity\User $user */
            $user = $link->User ?: $this->repository('XF:User')->getGuestUser($link->username);

            $message = $link->title . "\n" . $link->description;

            $checker = $this->app->spam()->contentChecker();

            $checker->check($user, $message, [
                'permalink' => $this->app->router('public')->buildLink('canonical:link-directory', $link),
                'content_type' => 'ld_link'
            ]);

            $decision = $checker->getFinalDecision();
            switch ($decision)
            {
                case 'moderated':
                    $link->state = 'moderated';
                    break;

                case 'denied':
                    $checker->logSpamTrigger('ld_link', null);
                    $link->error(\XF::phrase('your_content_cannot_be_submitted_try_later'));
                    break;
            }
        }
	}

	protected function finalSetup()
	{
	}

	protected function _validate()
	{
		$this->finalSetup();

		$link = $this->link;

        $link->preSave();
		$errors = $link->getErrors();

		if ($this->performValidations)
		{
			if (!$link->prefix_id && $link->Category->require_prefix && $link->Category->getUsablePrefixes() && !$link->canMove())
			{
				$errors[] = \XF::phraseDeferred('please_select_a_prefix');
			}
		}

		return $errors;
	}

	protected function _save()
	{
        $link = $this->link;

        $link->save(true, false);

		if ($link->isVisible() && $this->alert && $link->user_id != \XF::visitor()->user_id)
		{
			/** @var \xenMade\LD\Repository\Link $linkRepo */
            $linkRepo = $this->repository('xenMade\LD:Link');
            $linkRepo->sendModeratorActionAlert($this->link, 'edit', $this->alertReason);
		}

		return $link;
	}
}