<?php

namespace xenMade\LD\Service\Link;

class Move extends \XF\Service\AbstractService
{
	protected $link;

	protected $alert = false;
	protected $alertReason = '';

	protected $notifyWatchers = false;

	protected $prefixId = null;

	protected $extraSetup = [];

	public function __construct(\XF\App $app, \xenMade\LD\Entity\Link $link)
	{
		parent::__construct($app);
		$this->link = $link;
	}

	public function getLink()
	{
		return $this->link;
	}

	public function setSendAlert($alert, $reason = null)
	{
		$this->alert = (bool)$alert;
		if ($reason !== null)
		{
			$this->alertReason = $reason;
		}
	}

	public function setPrefix($prefixId)
	{
		$this->prefixId = ($prefixId === null ? $prefixId : intval($prefixId));
	}

	public function setNotifyWatchers($value = true)
	{
		$this->notifyWatchers = (bool)$value;
	}

	public function addExtraSetup(callable $extra)
	{
		$this->extraSetup[] = $extra;
	}

	public function move(\xenMade\LD\Entity\Category $category)
	{
		$user = \XF::visitor();

        $link = $this->link;
		$oldCategory = $link->Category;

		$moved = ($link->category_id != $category->category_id);

		foreach ($this->extraSetup AS $extra)
		{
			call_user_func($extra, $link, $category);
		}

        $link->category_id = $category->category_id;
		if ($this->prefixId !== null)
		{
            $link->prefix_id = $this->prefixId;
		}

		if (!$link->preSave())
		{
			throw new \XF\PrintableException($link->getErrors());
		}

		$db = $this->db();
		$db->beginTransaction();

        $link->save(true, false);

		$db->commit();

		if ($moved && $link->isVisible() && $this->alert && $link->user_id != $user->user_id)
		{
			/** @var \xenMade\LD\Repository\Link $linkRepo */
            $linkRepo = $this->repository('xenMade\LD:Link');
            $linkRepo->sendModeratorActionAlert($this->link, 'move', $this->alertReason);
		}

		if ($moved && $this->notifyWatchers)
		{
			/** @var \xenMade\LD\Service\Link\Notify $notifier */
			$notifier = $this->service('xenMade\LD:Link\Notify', $link, 'add');

			if ($oldCategory)
			{
				$notifier->skipUsersWatchingCategory($oldCategory);
			}

			$notifier->notifyAndEnqueue(3);
		}

		return $moved;
	}
}