<?php

namespace xenMade\LD\Service\Link;

use XF\Service\AbstractNotifier;

class Notify extends AbstractNotifier
{
	protected $link;

	protected $actionType;

	public function __construct(\XF\App $app, \xenMade\LD\Entity\Link $link, $actionType)
	{
		parent::__construct($app);

		switch ($actionType)
		{
			case 'add':
				break;

			default:
				throw new \InvalidArgumentException("Unknown action type '$actionType'");
		}

		$this->actionType = $actionType;
		$this->link = $link;
	}

	public static function createForJob(array $extraData)
	{
		$link = \XF::app()->find('xenMade\LD:Link', $extraData['linkId'], ['Category']);
		if (!$link)
		{
			return null;
		}

		return \XF::service('xenMade\LD:Link\Notify', $link, $extraData['actionType']);
	}

	protected function getExtraJobData()
	{
		return [
			'linkId' => $this->link->link_id,
			'actionType' => $this->actionType
		];
	}

	protected function loadNotifiers()
	{
		return [
			'categoryWatch' => $this->app->notifier('xenMade\LD:Link\CategoryWatch', $this->link, $this->actionType),
		];
	}

	protected function loadExtraUserData(array $users)
	{
		$permCombinationIds = [];
		foreach ($users AS $user)
		{
			$id = $user->permission_combination_id;
			$permCombinationIds[$id] = $id;
		}

		$this->app->permissionCache()->cacheMultipleContentPermsForContent($permCombinationIds, 'ld_category', $this->link->category_id);
	}

	protected function canUserViewContent(\XF\Entity\User $user)
	{
		return \XF::asVisitor($user, function() { return $this->link->canView(); });
	}

	public function skipUsersWatchingCategory(\xenMade\LD\Entity\Category $category)
	{
		$checkCategories = array_keys($category->breadcrumb_data);
		$checkCategories[] = $category->category_id;

		$db = $this->db();

		$watchers = $db->fetchAll("
			SELECT user_id, send_alert, send_email
			FROM xf_xenmade_ld_category_watch
			WHERE category_id IN (" . $db->quote($checkCategories) . ")
				AND (category_id = ? OR include_children > 0)
				AND (send_alert = 1 OR send_email = 1)
		", $category->category_id);

		foreach ($watchers AS $watcher)
		{
			if ($watcher['send_alert'])
			{
				$this->setUserAsAlerted($watcher['user_id']);
			}
			if ($watcher['send_email'])
			{
				$this->setUserAsEmailed($watcher['user_id']);
			}
		}
	}
}