<?php

namespace xenMade\LD\Service\Link;


class Preparer extends \XF\Service\AbstractService
{
    protected $link;

    /**
     * @var \xenMade\LD\Service\Link\Backlink
     */
    protected $backlinkChecker;

	public function __construct(\XF\App $app, \xenMade\LD\Entity\Link $link)
	{
		parent::__construct($app);
		$this->link = $link;

        $this->backlinkChecker = $this->service('xenMade\LD:Link\Backlink', $this->link);
    }

    public function setUrl($url)
    {
        if($this->link->url != $url)
        {
            $this->checkLinkExists($url);
        }

        $this->link->url = $url;

        $this->checkBlackList($url);
    }

    public function setBacklink()
    {
        $this->backlinkChecker->backlinkChecker();
    }

    protected function checkBlackList($url)
    {
        $blacklists = preg_split('#(\r\n|\n|\r|\s+|,)#s', trim(\XF::options()->ldDomainBlacklist), -1, PREG_SPLIT_NO_EMPTY);

        $pieces = @parse_url($url);
        $url = (isset($pieces['host']) ? $pieces['host'] : '');

        if(!$url)
        {
            return false;
        }

        if(!empty($blacklists))
        {
            foreach($blacklists as $blackdomain)
            {
                if(strpos($blackdomain, '*.') !== false)
                {
                    if($this->domainWildcard($blackdomain, $url))
                    {
                        return $this->link->error(\XF::phraseDeferred('ld_this_url_is_not_allowed'), 'url');
                    }
                }
                else
                {
                    if($blackdomain == $url)
                    {
                        return $this->link->error(\XF::phraseDeferred('ld_this_url_is_not_allowed'), 'url');
                    }
                }
            }
        }

        return false;
    }

    protected function checkLinkExists($url)
    {
        // Prüfe ob Link nur Domain ohne Path
        $urlSegements = @parse_url($url);

        if(isset($urlSegements['path']) && $urlSegements['path'] == '/')
        {
            $url = rtrim($url,'/');
        }

        $urlResult = $this->db()->fetchRow('SELECT * FROM xf_xenmade_ld_link WHERE url = ?', $url);

        if($urlResult)
        {
            return $this->link->error(\XF::phraseDeferred('ld_this_url_exists'), 'url');
        }

        return false;
    }

    protected function domainWildcard($domain, $url)
    {
        $domain = str_replace('*.', '', $domain);
        $pattern = preg_quote($domain, '/');

        return preg_match('/^(.*\.)?' . $pattern .'/i', $url, $matches);
    }
}