<?php

namespace xenMade\LD\Service\Link;

class Rating extends \XF\Service\AbstractService
{
	use \XF\Service\ValidateAndSavableTrait;

	protected $link;
	protected $rating;

	protected $mimiumTextLength = 0;

	protected $sendAlert = true;

	public function __construct(\XF\App $app, \xenMade\LD\Entity\Link $link)
	{
		parent::__construct($app);

		$this->link = $link;
		$this->rating = $this->setupRating();

		$this->mimiumTextLength = $app->options()->ldRatingReviewMinLength;
	}

	protected function setupRating()
	{
		$link = $this->link;

		$rating = $this->em()->create('xenMade\LD:LinkRating');
		$rating->link_id = $link->link_id;
		$rating->user_id = \XF::visitor()->user_id;
        $rating->username = \XF::visitor()->username;

		return $rating;
	}

	public function getLink()
	{
		return $this->link;
	}

	public function getRating()
	{
		return $this->rating;
	}

	public function setRating($rating, $message = '')
	{
		$this->rating->rating = $rating;
		$this->rating->message = $message;
	}

	public function setRateRequirements($minLength = null)
	{
		if ($minLength !== null)
		{
			$minLength = max(0, intval($minLength));
			$this->mimiumTextLength = $minLength;
		}
	}

	public function checkForSpam()
	{
		$rating = $this->rating;

		if (
			!\XF::visitor()->isSpamCheckRequired()
			|| !strlen($this->rating->message)
			|| $this->rating->getErrors()
		)
		{
			return;
		}

		/** @var \XF\Entity\User $user */
		$user = $rating->User;

		$message = $rating->message;

		$checker = $this->app->spam()->contentChecker();
		$checker->check($user, $message, [
			'permalink' => $this->app->router('public')->buildLink('canonical:link-directory', $rating->Link),
			'content_type' => 'ld_link_rating'
		]);

		$decision = $checker->getFinalDecision();
		switch ($decision)
		{
			case 'moderated':
			case 'denied':
				$checker->logSpamTrigger('ld_link_rating', null);
				$rating->error(\XF::phrase('your_content_cannot_be_submitted_try_later'));
				break;
		}
	}

	protected function _validate()
	{
		$rating = $this->rating;

		$rating->preSave();
		$errors = $rating->getErrors();

		if (utf8_strlen($rating->message) < $this->mimiumTextLength)
		{
			$errors['message'] = \XF::phrase(
				'ld_your_review_must_be_at_least_x_characters',
				['min' => $this->mimiumTextLength]
			);
		}

		if (!$rating->rating)
		{
			$errors['rating'] = \XF::phrase('ld_please_select_star_rating');
		}

		return $errors;
	}

	protected function _save()
	{
		$rating = $this->rating;

		$existing = $this->link->Rating[$rating->user_id];

		if ($existing)
		{
			$existing->delete();
		}

		$rating->save(true, false);

		if ($this->sendAlert && \XF::visitor()->user_id != $rating->user_id)
		{
			$this->repository('xenMade\LD:LinkRating')->sendRatingAlertToLinkAuthor($rating);
		}

		return $rating;
	}
}