<?php

namespace xenMade\LD\Service\Link;

class Thread extends \XF\Service\AbstractService
{
    protected $link;

	public function __construct(\XF\App $app, \xenMade\LD\Entity\Link $link)
	{
		parent::__construct($app);

        $this->link = $link;
	}

    public function setupThreadCreation(\xenMade\LD\Entity\Link $link)
    {
        $forum = $link->Category->ThreadForum;

        /** @var \XF\Service\Thread\Creator $creator */
        $creator = $this->service('XF:Thread\Creator', $forum);
        $creator->setIsAutomated();

        $bbDescription = $this->app->bbCode()->render($link->description, 'bbCodeClean', 'post', null);

        $imageReplace = '';

        if($link->image_date)
        {
            $imageLink = $link->getImage();

            if($imageLink)
            {
                $imageReplace = '[img]' . $link->getImage(true) . '[/img]' . "\n";
            }
        }

        $threadMessage = \XF::phrase('ld_link_thread_message', [
            'title' => $link->title,
            'username' => $link->User ? $link->User->username : $link->username,
            'image' => $imageReplace,
            'description' => $bbDescription,
            'link' => $this->app->router('public')->buildLink('canonical:link-directory', $this->link)
        ])
            ->render('raw');

        $creator->setContent($link->title, $threadMessage, false);
        $creator->setPrefix($link->Category->thread_prefix_id);

        $thread = $creator->getThread();
        $thread->bulkSet([
            'discussion_type' => 'ld_link',
            'discussion_state' => $this->link->state
        ]);

        return $creator;
    }

    public function afterThreadCreated(\XF\Entity\Thread $thread)
    {
        $this->repository('XF:Thread')->markThreadReadByVisitor($thread);
        $this->repository('XF:ThreadWatch')->autoWatchThread($thread, \XF::visitor(), true);
    }
}