<?php

namespace xenMade\LD\Service\Link;

class Upgrade extends \XF\Service\AbstractService
{
    protected $link;

    protected $purchaseRequestKey;

    protected $ugrade;

    protected $activeUpgrade;

    protected $ignoreUnpurchasable = false;

    protected $endDate;

    protected $extraData = [];

    protected $finalSetup = false;

    public function __construct(\XF\App $app, \xenMade\LD\Entity\LinkUpgrade $upgrade, \xenMade\LD\Entity\Link $link)
    {
        parent::__construct($app);

        $this->link = $link;
        $this->setUpgrade($upgrade);
    }

    public function getLink()
    {
        return $this->link;
    }

    public function setPurchaseRequestKey($purchaseRequestKey)
    {
        $this->purchaseRequestKey = $purchaseRequestKey;
    }

    protected function setUpgrade(\xenMade\LD\Entity\LinkUpgrade $upgrade)
    {
        $this->upgrade = $upgrade;
        $link = $this->link;

        $activeUpgrades = $upgrade->Active;
        $activeUpgrade = $activeUpgrades[$link->link_id];

        if (!$activeUpgrade)
        {
            $activeUpgrade = $this->em()->create('xenMade\LD:LinkUpgradeActive');
            $activeUpgrade->upgrade_id = $upgrade->upgrade_id;
            $activeUpgrade->link_id = $link->link_id;
            $activeUpgrade->start_date = time();
        }
        $this->activeUpgrade = $activeUpgrade;
    }

    public function getUpgrade()
    {
        return $this->upgrade;
    }

    public function getActiveUpgrade()
    {
        return $this->activeUpgrade;
    }

    public function ignoreUnpurchasable($ignoreUnpurchsable)
    {
        $this->ignoreUnpurchasable = $ignoreUnpurchsable;
    }

    public function setEndDate($endDate)
    {
        $this->endDate = intval($endDate);
    }

    public function setExtraData(array $extraData)
    {
        $this->extraData = $extraData;
    }

    protected function finalSetup()
    {
        $this->finalSetup = true;

        $active = $this->activeUpgrade;
        $upgrade = $this->upgrade;

        $endDate = $this->endDate;

        if ($active->upgrade_record_id)
        {
            if ($endDate === null)
            {
                if ($active->end_date == 0 || !$active->extra['length_unit'])
                {
                    $endDate = 0;
                }
                else
                {
                    $endDate = min(
                        pow(2,32) - 1,
                        strtotime('+' . $active->extra['length_amount'] . ' ' . $active->extra['length_unit'], $active->end_date)
                    );
                }
            }
        }
        else
        {
            if ($endDate === null)
            {
                if (!$upgrade->length_unit)
                {
                    $endDate = 0;
                }
                else
                {
                    $endDate = strtotime('+' . $upgrade->length_amount . ' ' . $upgrade->length_unit);
                }
            }

            $active->extra = array_merge([
                'cost_amount' => $upgrade->cost_amount,
                'cost_currency' => $upgrade->cost_currency,
                'length_amount' => $upgrade->length_amount,
                'length_unit' => $upgrade->length_unit
            ], $this->extraData);
        }

        $active->end_date = $endDate;

        if ($this->purchaseRequestKey)
        {
            $requestKey = $this->purchaseRequestKey;
            if (strlen($requestKey) > 32)
            {
                $requestKey = substr($requestKey, 0, 29) . '...';
            }

            $active->purchase_request_key = $requestKey;
        }
    }

    public function upgrade()
    {
        if (!$this->finalSetup)
        {
            $this->finalSetup();
        }

        $active = $this->activeUpgrade;
        $upgrade = $this->upgrade;
        $link = $this->link;

        if (!$active->upgrade_record_id)
        {
            if (!$upgrade->canPurchase($link) && !$this->ignoreUnpurchasable)
            {
                return false;
            }
        }

        $db = $this->db();
        $db->beginTransaction();

        if (!$active->save(true, false))
        {
            return false;
        }

        $link->set('is_vip', true);
        $link->save(false);

        /** @var \XF\Repository\UserAlert $alertRepo */
        //$alertRepo = $this->repository('XF:UserAlert');
        //$alertRepo->fastDeleteAlertsFromUser($user->user_id, 'user', $user->user_id, 'upgrade_end');

        $db->commit();

        return $active;
    }
}