<?php

namespace xenMade\LD\Service\LinkComment;

use xenMade\LD\Entity\Link;

class Creator extends \XF\Service\AbstractService
{
	use \XF\Service\ValidateAndSavableTrait;

	protected $link;
	protected $linkComment;
	protected $user;


	protected $preparer;

	public function __construct(\XF\App $app, Link $link)
	{
		parent::__construct($app);
		$this->setUser(\XF::visitor());

        $this->link = $link;
        $this->linkComment = $link->getNewLinkComment();
        $this->preparer = $this->service('xenMade\LD:LinkComment\Preparer', $this->linkComment);

		$this->setDefaults();
	}

	protected function setLink(Link $link)
	{
		$this->Link = $link;
		$this->linkComment = $link->getNewlinkComment();
		$this->preparer = $this->service('XF:LinkComment\Preparer', $this->linkComment);
	}

	public function getLink()
	{
		return $this->Link;
	}

	public function getLinkComment()
	{
		return $this->linkComment;
	}

	public function getlinkCommentPreparer()
	{
		return $this->preparer;
	}

	public function logIp($logIp)
	{
		$this->preparer->logIp($logIp);
	}

	protected function setUser(\XF\Entity\User $user)
	{
		$this->user = $user;
	}

	protected function setDefaults()
	{
		$this->linkComment->message_state = $this->linkComment->getNewContentState();
		$this->linkComment->user_id = $this->user->user_id;
		$this->linkComment->username = $this->user->username;
	}

	public function setContent($message, $format = true)
	{
		return $this->preparer->setMessage($message, $format);
	}

	public function checkForSpam()
	{
		if ($this->linkComment->message_state == 'visible' && $this->user->isSpamCheckRequired())
		{
			$this->preparer->checkForSpam();
		}
	}

	protected function finalSetup()
	{
		$this->linkComment->post_date = time();
	}

	protected function _validate()
	{
		$this->finalSetup();

		$this->linkComment->preSave();
		return $this->linkComment->getErrors();
	}

	protected function _save()
	{
		$linkComment = $this->linkComment;
		$linkComment->save();

		$this->preparer->afterInsert();

		return $linkComment;
	}

	public function sendNotifications()
	{
		if ($this->linkComment->isVisible())
		{
			/** @var \xenMade\LD\Service\LinkComment\Notifier $notifier */
			$notifier = $this->service('xenMade\LD:LinkComment\Notifier', $this->linkComment);
			$notifier->setNotifyMentioned($this->preparer->getMentionedUserIds());
			$notifier->notify();
		}
	}
}