<?php

namespace xenMade\LD\Service\LinkComment;

use xenMade\LD\Entity\LinkComment;

class Editor extends \XF\Service\AbstractService
{
	use \XF\Service\ValidateAndSavableTrait;

	protected $linkComment;
	protected $preparer;

	protected $alert = false;
	protected $alertReason = '';

	public function __construct(\XF\App $app, LinkComment $linkComment)
	{
		parent::__construct($app);
		$this->linkComment = $linkComment;
        $this->preparer = $this->service('xenMade\LD:LinkComment\Preparer', $linkComment);
	}

	public function getLinkComment()
	{
		return $this->linkComment;
	}

	public function getPreparer()
	{
		return $this->preparer;
	}

	public function setMessage($message, $format = true)
	{
		return $this->preparer->setMessage($message, $format);
	}

	public function setSendAlert($alert, $reason = null)
	{
		$this->alert = (bool)$alert;
		if ($reason !== null)
		{
			$this->alertReason = $reason;
		}
	}

	public function checkForSpam()
	{
		if ($this->linkComment->message_state == 'visible' && \XF::visitor()->isSpamCheckRequired())
		{
			$this->preparer->checkForSpam();
		}
	}

	protected function _validate()
	{
		$this->linkComment->preSave();
		return $this->linkComment->getErrors();
	}

	protected function _save()
	{
		$db = $this->db();
		$db->beginTransaction();

		$linkComment = $this->linkComment;
		$visitor = \XF::visitor();

        $linkComment->save(true, false);

		$this->preparer->afterUpdate();

		if ($linkComment->message_state == 'visible' && $this->alert && $linkComment->user_id != $visitor->user_id)
		{
			/** @var \xenMade\LD\Repository\LinkComment $linkCommentRepo */
            $linkCommentRepo = $this->repository('xenMade\LD:LinkComment');
            $linkCommentRepo->sendModeratorActionAlert($linkComment, 'edit', $this->alertReason);
		}

		$db->commit();

		return $linkComment;
	}
}