<?php

namespace xenMade\LD\Service\LinkComment;

use xenMade\LD\Entity\LinkComment;
use XF\Service\AbstractService;

class Notifier extends AbstractService
{
	protected $linkComment;

	protected $notifyInsert;
	protected $notifyMentioned = [];
    protected $notifyLinkOwner;
    protected $notifyOtherCommenters;

	protected $usersAlerted = [];

	public function __construct(\XF\App $app, LinkComment $linkComment)
	{
		parent::__construct($app);

		$this->linkComment = $linkComment;
	}

    public function getNotifyLinkOwner()
    {
        if ($this->notifyLinkOwner === null)
        {
            $this->notifyLinkOwner = [$this->linkComment->Link->user_id];
        }
        return $this->notifyLinkOwner;
    }

    public function getNotifyOtherCommenters()
    {
        if ($this->notifyOtherCommenters === null && $this->linkComment->Link)
        {
            /** @var \xenMade\LD\Repository\LinkComment $repo */
            $repo = $this->repository('xenMade\LD:LinkComment');
            $comments = $repo->findCommentsOnLink($this->linkComment->Link, ['visibility' => false])
                ->where('message_state', 'visible')
                ->fetch();

            $this->notifyOtherCommenters = $comments->pluckNamed('user_id');
        }
        return $this->notifyOtherCommenters;
    }

	public function setNotifyMentioned(array $mentioned)
	{
		$this->notifyMentioned = array_unique($mentioned);
	}

	public function getNotifyMentioned()
	{
		return $this->notifyMentioned;
	}

	public function notify()
	{
		$notifiableUsers = $this->getUsersForNotification();

        $linkUserId = $this->getNotifyLinkOwner();


        foreach ($linkUserId AS $userId)
        {
            if (isset($notifiableUsers[$userId]))
            {
                $this->sendNotification($notifiableUsers[$userId], 'insert');
            }
        }

        $otherCommenters = $this->getNotifyOtherCommenters();
        foreach ($otherCommenters AS $userId)
        {
            if (isset($notifiableUsers[$userId]))
            {
                $this->sendNotification($notifiableUsers[$userId], 'insert');
            }
        }

		$mentionUsers = $this->getNotifyMentioned();
		foreach ($mentionUsers AS $userId)
		{
			if (isset($notifiableUsers[$userId]))
			{
				$this->sendNotification($notifiableUsers[$userId], 'mention');
			}
		}
	}

	protected function getUsersForNotification()
	{
        $userIds = array_merge(
            $this->getNotifyLinkOwner(),
            $this->getNotifyMentioned(),
            $this->getNotifyOtherCommenters()
        );

		$linkComment = $this->linkComment;
		$users = $this->app->em()->findByIds('XF:User', $userIds, ['Profile', 'Option']);
		if (!$users->count())
		{
			return [];
		}

		$users = $users->toArray();
		foreach ($users AS $id => $user)
		{
			/** @var \XF\Entity\User $user */
			$canView = \XF::asVisitor($user, function() use ($linkComment) { return $linkComment->canView(); });
			if (!$canView)
			{
				unset($users[$id]);
			}
		}

		return $users;
	}

	protected function sendNotification(\XF\Entity\User $user, $action)
	{
		$linkComment = $this->linkComment;
		if ($user->user_id == $linkComment->user_id)
		{
			return false;
		}

		if (empty($this->usersAlerted[$user->user_id]))
		{
			/** @var \XF\Repository\UserAlert $alertRepo */
			$alertRepo = $this->app->repository('XF:UserAlert');
			if ($alertRepo->alert($user, $linkComment->user_id, $linkComment->username, 'ld_link_comment', $linkComment->comment_id, $action))
			{
				$this->usersAlerted[$user->user_id] = true;
				return true;
			}
		}

		return false;
	}

}