<?php

namespace xenMade\LD\Service\LinkComment;

use xenMade\LD\Entity\LinkComment;

class Preparer extends \XF\Service\AbstractService
{
	protected $linkComment;

	protected $logIp = true;

	protected $mentionedUsers = [];

	public function __construct(\XF\App $app, LinkComment $linkComment)
	{
		parent::__construct($app);
		$this->linkComment = $linkComment;
	}

	public function getLinkComment()
	{
		return $this->linkComment;
	}

	public function logIp($logIp)
	{
		$this->logIp = $logIp;
	}

	public function getMentionedUsers($limitPermissions = true)
	{
		if ($limitPermissions && $this->linkComment)
		{
			/** @var \XF\Entity\User $user */
			$user = $this->linkComment->User ?: $this->repository('XF:User')->getGuestUser();
			return $user->getAllowedUserMentions($this->mentionedUsers);
		}
		else
		{
			return $this->mentionedUsers;
		}
	}

	public function getMentionedUserIds($limitPermissions = true)
	{
		return array_keys($this->getMentionedUsers($limitPermissions));
	}

	public function setMessage($message, $format = true)
	{
		$preparer = $this->getStructuredTextPreparer($format);
		$preparer->setConstraint('maxLength', $this->app->options()->ldlinkCommentMaxLength);
		$this->linkComment->message = $preparer->prepare($message);

		$this->mentionedUsers = $preparer->getMentionedUsers();

		return $preparer->pushEntityErrorIfInvalid($this->linkComment);
	}

	/**
	 * @param bool $format
	 *
	 * @return \XF\Service\StructuredText\Preparer
	 */
	protected function getStructuredTextPreparer($format = true)
	{
		/** @var \XF\Service\StructuredText\Preparer $preparer */
		$preparer = $this->service('XF:StructuredText\Preparer', 'ld_link_comment');
		if (!$format)
		{
			$preparer->disableAllFilters();
		}

		return $preparer;
	}

	public function checkForSpam()
	{
		$linkComment = $this->linkComment;

		/** @var \XF\Entity\User $user */
		$user = $linkComment->User ?: $this->repository('XF:User')->getGuestUser($linkComment->username);
		$message = $linkComment->message;

		$checker = $this->app->spam()->contentChecker();
		$checker->check($user, $message, [
			'content_type' => 'ld_link_comment'
		]);

		$decision = $checker->getFinalDecision();
		switch ($decision)
		{
			case 'moderated':
				$linkComment->message_state = 'moderated';
				break;

			case 'denied':
				$checker->logSpamTrigger('ld_link_comment', null);
				$linkComment->error(\XF::phrase('ld_your_link_comment_cannot_be_submitted_try_later'));
				break;
		}
	}

	public function afterInsert()
	{
		if ($this->logIp)
		{
			$ip = ($this->logIp === true ? $this->app->request()->getIp() : $this->logIp);
			$this->writeIpLog($ip);
		}

		$checker = $this->app->spam()->contentChecker();
		$checker->logSpamTrigger('ld_link_comment', $this->linkComment->comment_id);
	}

	public function afterUpdate()
	{
		$checker = $this->app->spam()->contentChecker();
		$checker->logSpamTrigger('ld_link_comment', $this->linkComment->comment_id);
	}

	protected function writeIpLog($ip)
	{
		$linkComment = $this->linkComment;
		if (!$linkComment->user_id)
		{
			return;
		}

		/** @var \XF\Repository\IP $ipRepo */
		$ipRepo = $this->repository('XF:Ip');
		$ipEnt = $ipRepo->logIp($linkComment->user_id, $ip, 'ld_link_comment', $linkComment->comment_id);
		if ($ipEnt)
		{
			$linkComment->fastUpdate('ip_id', $ipEnt->ip_id);
		}
	}
}