<?php

namespace xenMade\LD;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;

class Setup extends AbstractSetup
{
    use StepRunnerInstallTrait;
    use StepRunnerUpgradeTrait;
    use StepRunnerUninstallTrait;


    public function installStep1()
    {
        $this->schemaManager()->createTable('xf_xenmade_ld_category', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('category_id', 'int')->autoIncrement();
            $table->addColumn('title', 'varchar', 100);
            $table->addColumn('description', 'text');
            $table->addColumn('link_count', 'int')->setDefault(0);
            $table->addColumn('last_update', 'int')->setDefault(0);
            $table->addColumn('last_link_title', 'varchar', 100)->setDefault('');
            $table->addColumn('last_link_id', 'int')->setDefault(0);
            $table->addColumn('field_cache', 'blob');
            $table->addColumn('prefix_cache', 'blob');
            $table->addColumn('require_prefix', 'tinyint')->setDefault(0);
            $table->addColumn('allow_link', 'tinyint')->setDefault(1);
            $table->addColumn('always_moderate_create', 'tinyint')->setDefault(0);
            $table->addColumn('always_moderate_update', 'tinyint')->setDefault(0);
            $table->addColumn('min_tags', 'int')->setDefault(0);
            $table->addColumn('parent_category_id', 'int')->setDefault(0);
            $table->addColumn('display_order', 'int')->setDefault(1);
            $table->addColumn('lft', 'int')->setDefault(0);
            $table->addColumn('rgt', 'int')->setDefault(0);
            $table->addColumn('depth', 'int')->setDefault(0);
            $table->addColumn('breadcrumb_data', 'blob');
            $table->addColumn('backlink', 'tinytext');
            $table->addColumn('backlink_location', 'smallint', 1)->setDefault(0);
            $table->addColumn('thread_node_id', 'int')->setDefault(0);
            $table->addColumn('thread_prefix_id', 'int')->setDefault(0);

            $table->addPrimaryKey('category_id');
            $table->addKey(['parent_category_id', 'lft']);
            $table->addKey(['lft', 'rgt']);
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_category_field', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('category_id', 'int');
            $table->addColumn('field_id', 'varchar', 25);

            $table->addPrimaryKey(['category_id', 'field_id']);
            $table->addKey('category_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_category_prefix', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('category_id', 'int');
            $table->addColumn('prefix_id', 'int');
            $table->addPrimaryKey(['category_id', 'prefix_id']);

            $table->addPrimaryKey(['category_id', 'prefix_id']);
            $table->addKey('prefix_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_category_watch', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('user_id', 'int');
            $table->addColumn('category_id', 'int');
            $table->addColumn('notify_on', 'enum')->values(['', 'add']);
            $table->addColumn('send_alert', 'tinyint')->setDefault(0);
            $table->addColumn('send_email', 'tinyint')->setDefault(0);
            $table->addColumn('include_children', 'tinyint')->setDefault(0);

            $table->addPrimaryKey(['category_id', 'user_id']);
            $table->addKey(['category_id', 'notify_on'], 'category_id_notify_on');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('link_id', 'int')->autoIncrement();
            $table->addColumn('title', 'varchar', 100);
            $table->addColumn('description', 'text');
            $table->addColumn('url', 'varchar', 2083);
            $table->addColumn('affiliate_url', 'varchar', 2083);
            $table->addColumn('backlink_location', 'varchar', 2083);
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 50);
            $table->addColumn('state', 'enum')->values(['visible', 'moderated', 'deleted']);
            $table->addColumn('insert_date', 'int')->setDefault(0);
            $table->addColumn('last_update', 'int')->setDefault(0);
            $table->addColumn('image_date', 'int')->setDefault(0);
            $table->addColumn('backlink_check', 'int')->setDefault(0);
            $table->addColumn('status_check', 'int')->setDefault(0);
            $table->addColumn('status_fails', 'int')->setDefault(0);
            $table->addColumn('warning_id', 'int')->setDefault(0);
            $table->addColumn('warning_message', 'varchar', 255)->setDefault('');
            $table->addColumn('ip_id', 'int')->setDefault(0);
            $table->addColumn('category_id', 'int')->setDefault(0);
            $table->addColumn('thread_id', 'int')->setDefault(0);
            $table->addColumn('click_count', 'int')->setDefault(0);
            $table->addColumn('comment_count', 'int')->setDefault(0);
            $table->addColumn('rating_count', 'int')->setDefault(0);
            $table->addColumn('rating_sum', 'int')->setDefault(0);
            $table->addColumn('rating_avg', 'float')->setDefault(0);
            $table->addColumn('rating_weighted', 'float')->setDefault(0);
            $table->addColumn('custom_fields', 'blob');
            $table->addColumn('tags', 'blob');
            $table->addColumn('prefix_id', 'int')->setDefault(0);
            $table->addColumn('is_vip', 'int')->setDefault(0);
            $table->addColumn('likes', 'int')->setDefault(0);
            $table->addColumn('like_users', 'blob');

            $table->addPrimaryKey('link_id');
            $table->addKey('user_id');
            $table->addKey('insert_date');
            $table->addKey('last_update');
            $table->addKey('image_date');
            $table->addKey('category_id');
            $table->addKey('prefix_id');
            $table->addKey('is_vip');
            $table->addKey(['category_id', 'last_update'], 'category_last_update');
            $table->addKey(['category_id', 'rating_weighted'], 'category_rating_weighted');
            $table->addKey('rating_weighted');
            $table->addKey(['user_id', 'last_update']);

        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_comment', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('comment_id', 'int')->autoIncrement();
            $table->addColumn('link_id', 'int');
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 50);
            $table->addColumn('post_date', 'int')->setDefault(0);
            $table->addColumn('message', 'text');
            $table->addColumn('ip_id', 'int')->setDefault(0);
            $table->addColumn('message_state', 'enum')->values(['visible', 'moderated', 'deleted']);
            $table->addColumn('likes', 'int')->setDefault(0);
            $table->addColumn('like_users', 'blob');
            $table->addColumn('warning_id', 'int')->setDefault(0);
            $table->addColumn('warning_message', 'text');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_field', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('field_id', 'varchar', 25);
            $table->addColumn('display_order', 'int')->setDefault(1);
            $table->addColumn('field_type', 'enum')->values(['textbox', 'textarea', 'bbcode', 'select', 'radio', 'checkbox', 'multiselect', 'date', 'stars', 'color']);
            $table->addColumn('field_choices', 'blob');
            $table->addColumn('match_type', 'enum')->values(['none', 'number', 'alphanumeric', 'email', 'url', 'color', 'date', 'regex', 'callback', 'validator']);
            $table->addColumn('match_params', 'blob');
            $table->addColumn('max_length', 'int')->setDefault(0);
            $table->addColumn('required', 'tinyint')->setDefault(0);
            $table->addColumn('display_template', 'text');
            $table->addColumn('display_group', 'enum')->values(['above_main', 'above_info', 'below_info']);

            $table->addPrimaryKey('field_id');
            $table->addKey(['display_group', 'display_order'], 'display_group_order');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_field_value', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('link_id', 'int');
            $table->addColumn('field_id', 'varchar', 25);
            $table->addColumn('field_value', 'text');

            $table->addPrimaryKey(['link_id', 'field_id']);
            $table->addKey('field_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_prefix', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('prefix_id', 'int')->autoIncrement();
            $table->addColumn('prefix_group_id', 'int')->setDefault(0);
            $table->addColumn('display_order', 'int')->setDefault(1);
            $table->addColumn('materialized_order', 'int')->setDefault(0);
            $table->addColumn('css_class', 'varchar', 50)->setDefault('label label--primary');
            $table->addColumn('allowed_user_group_ids', 'blob');

            $table->addKey('materialized_order');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_prefix_group', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('prefix_group_id', 'int')->autoIncrement();
            $table->addColumn('display_order', 'int')->setDefault(1);
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_rating', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('rating_id', 'int')->autoIncrement();
            $table->addColumn('link_id', 'int');
            $table->addColumn('user_id', 'int');
            $table->addColumn('username', 'varchar', 50);
            $table->addColumn('rating', 'int');
            $table->addColumn('rating_date', 'int')->setDefault(0);
            $table->addColumn('message', 'text');
            $table->addColumn('author_response', 'text');
            $table->addColumn('rating_state', 'enum')->values(['visible', 'deleted']);
            $table->addColumn('warning_id', 'int')->setDefault(0);

            $table->addUniqueKey(['link_id', 'user_id']);
            $table->addKey('link_id');
            $table->addKey('user_id');
            $table->addKey('rating_date');
            $table->addKey(['link_id', 'rating_date']);
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_watch', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('user_id', 'int');
            $table->addColumn('link_id', 'int');
            $table->addColumn('email_subscribe', 'tinyint')->setDefault(0);

            $table->addPrimaryKey(['link_id', 'user_id']);
            $table->addKey(['link_id', 'email_subscribe']);
        });



        $this->schemaManager()->createTable('xf_xenmade_ld_link_upgrade_active', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('upgrade_record_id', 'int');
            $table->addColumn('link_id', 'int');
            $table->addColumn('purchase_request_key', 'varbinary', 32)->nullable();
            $table->addColumn('upgrade_id', 'int');
            $table->addColumn('extra', 'mediumblob');
            $table->addColumn('start_date', 'int');
            $table->addColumn('end_date', 'int')->setDefault(0);

            $table->addPrimaryKey('upgrade_record_id');
            $table->addKey('end_date');
            $table->addKey('link_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_upgrade_expired', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('upgrade_record_id', 'int');
            $table->addColumn('link_id', 'int');
            $table->addColumn('purchase_request_key', 'varbinary', 32)->nullable();
            $table->addColumn('upgrade_id', 'int');
            $table->addColumn('extra', 'mediumblob');
            $table->addColumn('start_date', 'int');
            $table->addColumn('end_date', 'int')->setDefault(0);
            $table->addColumn('original_end_date', 'int')->setDefault(0);

            $table->addPrimaryKey('upgrade_record_id');
            $table->addKey('end_date');
            $table->addKey('link_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_category_upgrade', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('category_id', 'int');
            $table->addColumn('upgrade_id', 'int');

            $table->addPrimaryKey(['category_id', 'upgrade_id']);
            $table->addKey('category_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_upgrade', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('upgrade_id', 'int')->autoIncrement();
            $table->addColumn('title', 'varchar', 50);
            $table->addColumn('description', 'text');
            $table->addColumn('display_order', 'int')->setDefault(0);
            $table->addColumn('recurring', 'tinyint', 3)->setDefault(0);
            $table->addColumn('cost_amount', 'decimal', '10,2');
            $table->addColumn('cost_currency', 'varchar', 3);
            $table->addColumn('length_amount', 'tinyint', 3);
            $table->addColumn('length_unit', 'enum')->values(['day','month','year',''])->setDefault('');
            $table->addColumn('can_purchase', 'tinyint', 3)->setDefault(1);
            $table->addColumn('payment_profile_ids', 'varbinary', 255)->setDefault('');

            $table->addKey('display_order');
        });


    }

    public function installStep2()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('ld_link_count', 'int')->setDefault(0);
            $table->addKey('ld_link_count');
        });
    }

    public function installStep3()
    {
        $this->db()->query("INSERT INTO `xf_xenmade_ld_category` (`title`, `description`, `field_cache`, `prefix_cache`, `breadcrumb_data`, `backlink`) VALUES ('Example category', '', '', '', '0x613a303a7b7d', '')");

        $this->db()->query("
			REPLACE INTO xf_purchasable
				(purchasable_type_id, purchasable_class, addon_id)
			VALUES
				('ld_link_upgrade', 'xenMade\\\\LD:LinkUpgrade', 'xenMade/LD')
		");

        foreach ($this->getDefaultWidgetSetup() AS $widgetKey => $widgetFn)
        {
            $widgetFn($widgetKey);
        }
    }





    public function upgrade20001Step1()
    {
        $this->db()->query('UPDATE xf_xenmade_ld_link SET rating_weighted = 0 WHERE rating_count = 0');
    }

    public function upgrade20002Step1()
    {
        $this->schemaManager()->createTable('xf_xenmade_ld_link_upgrade_active', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('upgrade_record_id', 'int');
            $table->addColumn('link_id', 'int');
            $table->addColumn('purchase_request_key', 'varbinary', 32)->nullable();
            $table->addColumn('upgrade_id', 'int');
            $table->addColumn('extra', 'mediumblob');
            $table->addColumn('start_date', 'int');
            $table->addColumn('end_date', 'int')->setDefault(0);

            $table->addPrimaryKey('upgrade_record_id');
            $table->addKey('end_date');
            $table->addKey('link_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_upgrade_expired', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('upgrade_record_id', 'int');
            $table->addColumn('link_id', 'int');
            $table->addColumn('purchase_request_key', 'varbinary', 32)->nullable();
            $table->addColumn('upgrade_id', 'int');
            $table->addColumn('extra', 'mediumblob');
            $table->addColumn('start_date', 'int');
            $table->addColumn('end_date', 'int')->setDefault(0);
            $table->addColumn('original_end_date', 'int')->setDefault(0);

            $table->addPrimaryKey('upgrade_record_id');
            $table->addKey('end_date');
            $table->addKey('link_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_category_upgrade', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('category_id', 'int');
            $table->addColumn('upgrade_id', 'int');

            $table->addPrimaryKey(['category_id', 'upgrade_id']);
            $table->addKey('category_id');
        });

        $this->schemaManager()->createTable('xf_xenmade_ld_link_upgrade', function(\XF\Db\Schema\Create $table)
        {
            $table->addColumn('upgrade_id', 'int')->autoIncrement();
            $table->addColumn('title', 'varchar', 50);
            $table->addColumn('description', 'text');
            $table->addColumn('display_order', 'int')->setDefault(0);
            $table->addColumn('recurring', 'tinyint', 3)->setDefault(0);
            $table->addColumn('cost_amount', 'decimal', '10,2');
            $table->addColumn('cost_currency', 'varchar', 3);
            $table->addColumn('length_amount', 'tinyint', 3);
            $table->addColumn('length_unit', 'enum')->values(['day','month','year',''])->setDefault('');
            $table->addColumn('can_purchase', 'tinyint', 3)->setDefault(1);
            $table->addColumn('payment_profile_ids', 'varbinary', 255)->setDefault('');

            $table->addKey('display_order');
        });

        $this->db()->query("
			REPLACE INTO xf_purchasable
				(purchasable_type_id, purchasable_class, addon_id)
			VALUES
				('ld_link_upgrade', 'xenMade\\\\LD:LinkUpgrade', 'xenMade/LD')
		");


        $sm = $this->schemaManager();
        $sm->alterTable('xf_xenmade_ld_link', function(Alter $table)
        {
            $table->addColumn('backlink_check', 'int');
        });
    }

    public function upgrade20003Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_xenmade_ld_category', function(Alter $table)
        {
            $table->addColumn('backlink', 'tinytext');
        });
    }

    public function upgrade20006Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_xenmade_ld_category', function(Alter $table)
        {
            $table->addColumn('thread_node_id', 'int')->setDefault(0);
            $table->addColumn('thread_prefix_id', 'int')->setDefault(0);
            $table->addColumn('backlink_location', 'smallint', 1)->setDefault(0)->after('backlink');
        });
    }

    public function upgrade20006Step2()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_xenmade_ld_link', function(Alter $table)
        {
            $table->addColumn('thread_id', 'int')->setDefault(0);
        });
    }

    public function upgrade20006Step3()
    {
        foreach ($this->getDefaultWidgetSetup() AS $widgetKey => $widgetFn)
        {
            $widgetFn($widgetKey);
        }
    }

    public function upgrade20007Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_xenmade_ld_link', function(Alter $table)
        {
            $table->addColumn('backlink_location', 'varchar', 2083)->after('affiliate_url');
            $table->addColumn('status_check', 'int')->setDefault(0)->after('backlink_check');
            $table->addColumn('status_fails', 'int')->setDefault(0)->after('status_check');
        });
    }

    public function upgrade20016Step1()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_xenmade_ld_link_rating', function(Alter $table)
        {
            $table->addColumn('username', 'varchar', 50);
        });
    }



    public function uninstallStep1()
    {
        $this->schemaManager()->dropTable('xf_xenmade_ld_category');
        $this->schemaManager()->dropTable('xf_xenmade_ld_category_field');
        $this->schemaManager()->dropTable('xf_xenmade_ld_category_prefix');
        $this->schemaManager()->dropTable('xf_xenmade_ld_category_watch');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link_comment');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link_field');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link_field_value');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link_prefix');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link_prefix_group');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link_rating');
        $this->schemaManager()->dropTable('xf_xenmade_ld_link_watch');
    }

    public function uninstallStep2()
    {
        $sm = $this->schemaManager();

        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->dropColumns('ld_link_count');
        });
    }

    public function uninstallStep3()
    {
        $db = $this->db();

        $contentTypes = ['ld_link', 'ld_category', 'ld_link_comment', 'ld_link_rating'];
        $contentTypesQuoted = $db->quote($contentTypes);

        $db->beginTransaction();

        $contentTypeTables = [
            'xf_approval_queue',
            'xf_deletion_log',
            'xf_liked_content',
            'xf_moderator_log',
            'xf_news_feed',
            'xf_report',
            'xf_tag_content',
            'xf_user_alert'
        ];

        foreach ($contentTypeTables AS $table)
        {
            $db->delete($table, 'content_type IN (' . $contentTypesQuoted . ')');
        }

        $db->delete('xf_permission_cache_content', "content_type = 'ld_category'");
        $db->delete('xf_permission_entry', "permission_group_id = 'ld_link'");
        $db->delete('xf_permission_entry', "permission_group_id = 'ld_link_comment'");

        $db->commit();

        foreach ($this->getDefaultWidgetSetup() AS $widgetKey => $widgetFn)
        {
            $this->deleteWidget($widgetKey);
        }
    }



    public function postInstall(array &$stateChanges)
    {
        if ($this->applyDefaultPermissions())
        {
            $this->app->jobManager()->enqueueUnique(
                'permissionRebuild',
                'XF:PermissionRebuild',
                [],
                false
            );
        }

        /** @var \XF\Service\RebuildNestedSet $service */
        $service = \XF::service('XF:RebuildNestedSet', 'xenMade\LD:Category', ['parentField' => 'parent_category_id']);
        $service->rebuildNestedSetInfo();

        \XF::repository('xenMade\LD:LinkPrefix')->rebuildPrefixCache();
        \XF::repository('xenMade\LD:LinkField')->rebuildFieldCache();
    }



    protected function getDefaultWidgetSetup()
    {
        return [

            /// Overview Widgets
            'ld_latest_rating_ld_overview' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_latest_ratings',
                    [
                        'positions' => ['ld_overview_sidenav' => 200],
                        'options' => $options
                    ],
                    'Latest Rating'
                );
            },

            'ld_random_links_ld_overview' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_random_links',
                    [
                        'positions' => ['ld_overview_sidenav' => 200],
                        'options' => $options
                    ],
                    'Random Links'
                );
            },

            'ld_top_links_ld_overview' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_top_links',
                    [
                        'positions' => ['ld_overview_sidenav' => 200],
                        'options' => $options
                    ],
                    'Top Links'
                );
            },

            'ld_new_links_ld_overview' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_new_links',
                    [
                        'positions' => ['ld_overview_sidenav' => 200],
                        'options' => $options
                    ],
                    'New Links'
                );
            },

            'ld_vip_links_ld_overview' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_vip_links',
                    [
                        'positions' => ['ld_overview_sidenav' => 200],
                        'options' => $options
                    ],
                    'VIP Links'
                );
            },


            // Category Widgets
            'ld_latest_rating_ld_category' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_latest_ratings',
                    [
                        'positions' => ['ld_category_sidenav' => 200],
                        'options' => $options
                    ],
                    'Latest Rating'
                );
            },

            'ld_random_links_ld_category' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_random_links',
                    [
                        'positions' => ['ld_category_sidenav' => 200],
                        'options' => $options
                    ],
                    'Random Links'
                );
            },

            'ld_top_links_ld_category' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_top_links',
                    [
                        'positions' => ['ld_category_sidenav' => 200],
                        'options' => $options
                    ],
                    'Top Links'
                );
            },

            'ld_new_links_ld_category' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_new_links',
                    [
                        'positions' => ['ld_category_sidenav' => 200],
                        'options' => $options
                    ],
                    'New Links'
                );
            },

            'ld_vip_links_ld_category' => function($key, array $options = [])
            {
                $options = array_replace([], $options);

                $this->createWidget(
                    $key,
                    'ld_vip_links',
                    [
                        'positions' => ['ld_category_sidenav' => 200],
                        'options' => $options
                    ],
                    'VIP Links'
                );
            }
        ];
    }

    protected function applyDefaultPermissions($previousVersion = false)
    {
        $applied = false;

        if ($previousVersion === false)
        {
            $this->applyGlobalPermission('ld_link', 'add', 'forum', 'postThread');
            $this->applyGlobalPermission('ld_link', 'approveUnapprove', 'forum', 'approveUnapprove');
            $this->applyGlobalPermission('ld_link', 'deleteAny', 'forum', 'deleteAnyPost');
            $this->applyGlobalPermission('ld_link', 'deleteAnyRating', 'forum', 'deleteAnyPost');
            $this->applyGlobalPermission('ld_link', 'deleteOwn', 'forum', 'deleteOwnPost');
            $this->applyGlobalPermission('ld_link', 'editAny', 'forum', 'editAnyPost');
            $this->applyGlobalPermission('ld_link', 'hardDeleteAny', 'forum', 'hardDeleteAnyPost');
            $this->applyGlobalPermission('ld_link', 'rate', 'forum', 'like');
            $this->applyGlobalPermission('ld_link', 'reassign', 'forum', 'editAnyPost');
            $this->applyGlobalPermission('ld_link', 'undelete', 'forum', 'undelete');
            $this->applyGlobalPermission('ld_link', 'updateOwn', 'forum', 'editOwnPost');
            $this->applyGlobalPermission('ld_link', 'view', 'general', 'viewNode');
            $this->applyGlobalPermission('ld_link', 'viewDeleted', 'forum', 'viewDeleted');
            $this->applyGlobalPermission('ld_link', 'viewModerated', 'forum', 'viewModerated');
            $this->applyGlobalPermission('ld_link', 'warn', 'general', 'warn');
            $this->applyGlobalPermission('ld_link', 'inlineMod', 'forum', 'inlineMod');
            $this->applyGlobalPermission('ld_link', 'manageAnyTag', 'forum', 'manageAnyTag');
            $this->applyGlobalPermission('ld_link', 'tagAnyLink', 'forum', 'tagAnyThread');
            $this->applyGlobalPermission('ld_link', 'tagOwnLink', 'forum', 'tagOwnThread');


            $this->applyGlobalPermission('ld_link_comment', 'approveUnapprove', 'forum', 'approveUnapprove');
            $this->applyGlobalPermission('ld_link_comment', 'deleteAny', 'forum', 'deleteAnyPost');
            $this->applyGlobalPermission('ld_link_comment', 'deleteOwn', 'forum', 'deleteOwnPost');
            $this->applyGlobalPermission('ld_link_comment', 'editAny', 'forum', 'editAnyPost');
            $this->applyGlobalPermission('ld_link_comment', 'editOwn', 'forum', 'editOwnPost');
            $this->applyGlobalPermission('ld_link_comment', 'hardDeleteAny', 'forum', 'hardDeleteAnyPost');
            $this->applyGlobalPermission('ld_link_comment', 'inlineMod', 'forum', 'inlineMod');
            $this->applyGlobalPermission('ld_link_comment', 'like', 'forum', 'like');
            $this->applyGlobalPermission('ld_link_comment', 'post', 'forum', 'postReply');
            $this->applyGlobalPermission('ld_link_comment', 'undelete', 'forum', 'undelete');
            $this->applyGlobalPermission('ld_link_comment', 'view', 'forum', 'viewContent');
            $this->applyGlobalPermission('ld_link_comment', 'viewDeleted', 'forum', 'viewDeleted');
            $this->applyGlobalPermission('ld_link_comment', 'viewModerated', 'forum', 'viewModerated');
            $this->applyGlobalPermission('ld_link_comment', 'warn', 'forum', 'warn');

            $applied = true;
        }

        return $applied;
    }
}