<?php

namespace xenMade\LD\Spam\Cleaner;

use XF\Spam\Cleaner\AbstractHandler;

class Link extends AbstractHandler
{
	public function canCleanUp(array $options = [])
	{
		return !empty($options['action_threads']);
	}

	public function cleanUp(array &$log, &$error = null)
	{
		$app = \XF::app();

		$links = $app->finder('xenMade\LD:Link')
			->where('user_id', $this->user->user_id)
			->fetch();

		if ($links->count())
		{
			$submitter = $app->container('spam.contentSubmitter');
			$submitter->submitSpam('ld_link', $links->keys());

			$deleteType = $app->options()->spamMessageAction == 'delete' ? 'hard' : 'soft';

			$log['ld_link'] = [
				'deleteType' => $deleteType,
				'linkIds' => []
			];

			foreach ($links AS $linkId => $link)
			{
				$log['ld_link']['linkIds'][] = $linkId;

				/** @var \xenMade\LD\Entity\Link $link */
                $link->setOption('log_moderator', false);
				if ($deleteType == 'soft')
				{
                    $link->softDelete();
				}
				else
				{
                    $link->delete();
				}
			}
		}

		return true;
	}

	public function restore(array $log, &$error = null)
	{
		if ($log['deleteType'] == 'soft')
		{
			$links = \XF::app()->finder('xenMade\LD:Link')
				->where('link_id', $log['linkIds'])
				->fetch();

			foreach ($links AS $link)
			{
				/** @var \xenMade\LD\Entity\Link $link */
                $link->setOption('log_moderator', false);
                $link->state = 'visible';
                $link->save();
			}
		}

		return true;
	}
}