<?php

namespace xenMade\LD\Spam\Cleaner;

use XF\Spam\Cleaner\AbstractHandler;

class LinkRating extends AbstractHandler
{
	public function canCleanUp(array $options = [])
	{
		return !empty($options['delete_messages']);
	}

	public function cleanUp(array &$log, &$error = null)
	{
		$app = \XF::app();

		$ratings = $app->finder('xenMade\LD:LinkRating')
			->where('user_id', $this->user->user_id)
			->fetch();

		if ($ratings->count())
		{
			$submitter = $app->container('spam.contentSubmitter');
			$submitter->submitSpam('ld_link_rating', $ratings->keys());

			$deleteType = $app->options()->spamMessageAction == 'delete' ? 'hard' : 'soft';

			$log['ld_link_rating'] = [
				'deleteType' => $deleteType,
				'ratingIds' => []
			];

			foreach ($ratings AS $ratingId => $rating)
			{
				$log['ld_link_rating']['ratingIds'][] = $ratingId;

				/** @var \xenMade\LD\Entity\LinkRating $rating */
				$rating->setOption('log_moderator', false);
				if ($deleteType == 'soft')
				{
					$rating->softDelete();
				}
				else
				{
					$rating->delete();
				}
			}
		}

		return true;
	}

	public function restore(array $log, &$error = null)
	{
		if ($log['deleteType'] == 'soft')
		{
			$ratings = \XF::app()->finder('xenMade\LD:LinkRating')
				->where('rating_id', $log['ratingIds'])
				->fetch();

			foreach ($ratings AS $rating)
			{
				/** @var \xenMade\LD\Entity\LinkRating $rating */
				$rating->setOption('log_moderator', false);
				$rating->rating_state = 'visible';
				$rating->save();
			}
		}

		return true;
	}
}