<?php

namespace xenMade\LD\Tag;

use XF\Mvc\Entity\Entity;
use XF\Tag\AbstractHandler;

class Link extends AbstractHandler
{
	public function getPermissionsFromContext(Entity $entity)
	{
		if ($entity instanceof \xenMade\LD\Entity\Link)
		{
            $link = $entity;
			$category = $link->Category;
		}
		else if ($entity instanceof \xenMade\LD\Entity\Category)
		{
            $link = null;
			$category = $entity;
		}
		else
		{
			throw new \InvalidArgumentException("Entity must be a link or category");
		}

		$visitor = \XF::visitor();

		if ($link)
		{
			if ($link->user_id == $visitor->user_id && $link->hasPermission('manageOthersTags'))
			{
				$removeOthers = true;
			}
			else
			{
				$removeOthers = $link->hasPermission('manageAnyTag');
			}

			$edit = $link->canEditTags();
		}
		else
		{
			$removeOthers = false;
			$edit = $category->canEditTags();
		}

		return [
			'edit' => $edit,
			'removeOthers' => $removeOthers,
			'minTotal' => $category->min_tags
		];
	}

	public function getContentDate(Entity $entity)
	{
		return $entity->insert_date;
	}

	public function getContentVisibility(Entity $entity)
	{
		return $entity->state == 'visible';
	}

	public function getTemplateData(Entity $entity, array $options = [])
	{
		return [
			'link' => $entity,
			'options' => $options
		];
	}

	public function getEntityWith($forView = false)
	{
		$get = ['Category'];
		if ($forView)
		{
			$get[] = 'User';

			$visitor = \XF::visitor();
			$get[] = 'Category.Permissions|' . $visitor->permission_combination_id;
		}

		return $get;
	}

	public function canUseInlineModeration(Entity $entity, &$error = null)
	{
		/** @var \xenMade\LD\Entity\Link $entity */
		return $entity->canUseInlineModeration($error);
	}
}