<?php

namespace xenMade\LD\Thumbnail;

abstract class AbstractThumbnail
{
	protected $app;
	protected $link;
	protected $error;

    /**
     * @var \XF\Http\Upload
     */
	protected $upload;

	public function __construct(\XF\App $app)
	{
		$this->app = $app;
	}

    abstract function getThumbnail();

    public function getThumbnailPath()
    {
        return sprintf('data://ld_thumbnail/%s/%d.jpg',
            floor($this->link->link_id / 1000),
            $this->link->link_id
        );
    }

    protected function saveThumbnail(\XF\Http\Upload $upload)
    {
        $this->setUpload($upload);

        $imageManager = $this->app->imageManager();

        try
        {
            $image = $imageManager->imageFromFile($this->getFileName());

            if ($image)
            {
                $this->resizeThumbnail($image);
                $image->save($this->getFileName());
            }
        }
        catch (\Exception $e)
        {
            \XF::logException($e, false, "Failed to save rhumbnail for link {$this->link->link_id}: ");
        }

        \XF\Util\File::copyFileToAbstractedPath($this->getFileName(), $this->getThumbnailPath());
    }

    protected function getFileName()
    {
        return $this->upload->getTempFile();
    }

    protected function resizeThumbnail(\XF\Image\AbstractDriver $image)
    {
        $targetSize = 192;

        if ($image->getHeight() != $targetSize || $image->getWidth() != $targetSize)
        {
            $image->resize($targetSize);
        }

        $newTempFile = \XF\Util\File::getTempFile();
        if ($newTempFile && $image->save($newTempFile))
        {
            return $newTempFile;
        }

        return false;
    }

    public function deleteThumbnail()
    {

    }

    public function getError()
    {
        return $this->error;
    }

    public function setError($errors)
    {
        return $this->error = $errors;
    }

    protected function setUpload($upload)
    {
        return $this->upload = $upload;
    }

    public function setLink(\xenMade\LD\Entity\Link $link)
    {
        return $this->link = $link;
    }

    public function getLink()
    {
        return $this->link;
    }
}