<?php

namespace xenMade\LD\Widget;

use XF\Widget\AbstractWidget;

class LatestRatings extends AbstractWidget
{
	protected $defaultOptions = [
		'limit' => 5
	];

	public function render()
	{
		$visitor = \XF::visitor();
		if (!method_exists($visitor, 'canViewLinks') || !$visitor->canViewLinks() || !$visitor->canViewLinkRatings())
		{
			return '';
		}

        $sourceCategoryIds = null;
		$options = $this->options;
		$limit = $options['limit'];

        $hasCategoryContext = (isset($this->contextParams['category']) && $this->contextParams['category'] instanceof \xenMade\LD\Entity\Category);

        if ($hasCategoryContext)
        {
            /** @var \xenMade\LD\Entity\Category $category */
            $category = $this->contextParams['category'];
            $viewableDescendents = $category->getViewableDescendants();
            $sourceCategoryIds = array_keys($viewableDescendents);
            $sourceCategoryIds[] = $category->category_id;
        }

        /** @var \xenMade\LD\Finder\LinkRating $finder */
		$finder = $this->repository('xenMade\LD:LinkRating')->findLatestRatings($sourceCategoryIds);
		$ratings = $finder->fetch(max($limit * 2, 10));

		/** @var \xenMade\LD\Entity\LinkRating $rating */
		foreach ($ratings AS $id => $rating)
		{
			if (!$rating->canView())
			{
				unset($ratings[$id]);
			}
		}

		$total = $ratings->count();
		$ratings = $ratings->slice(0, $limit);

		$viewParams = [
			'title' => $this->getTitle(),
			'ratings' => $ratings,
			'hasMore' => $total > $ratings->count()
		];
		return $this->renderer('ld_widget_latest_ratings', $viewParams);
	}

	public function verifyOptions(\XF\Http\Request $request, array &$options, &$error = null)
	{
		$options = $request->filter([
			'limit' => 'uint'
		]);
		if ($options['limit'] < 1)
		{
			$options['limit'] = 1;
		}

		return true;
	}
}