<?php

namespace xenMade\LD\Widget;

use XF\Widget\AbstractWidget;

class NewLink extends AbstractWidget
{
	protected $defaultOptions = [
		'limit' => 5
	];

    public function render()
    {
        $visitor = \XF::visitor();
        if (!method_exists($visitor, 'canViewLinks') || !$visitor->canViewLinks())
        {
            return '';
        }

        $sourceCategoryIds = null;
        $options = $this->options;
        $limit = $options['limit'];

        $hasCategoryContext = (isset($this->contextParams['category']) && $this->contextParams['category'] instanceof \xenMade\LD\Entity\Category);

        if ($hasCategoryContext)
        {
            /** @var \xenMade\LD\Entity\Category $category */
            $category = $this->contextParams['category'];
            $viewableDescendents = $category->getViewableDescendants();
            $sourceCategoryIds = array_keys($viewableDescendents);
            $sourceCategoryIds[] = $category->category_id;
        }


        /** @var \xenMade\LD\Finder\Link $finder */
        $finder = $this->repository('xenMade\LD:Link')->findNewLinks($sourceCategoryIds);
        $links = $finder->fetch(max($limit * 2, 10));

        /** @var \xenMade\LD\Entity\Link $link */
        foreach ($links AS $id => $link)
        {
            if (!$link->canView())
            {
                unset($links[$id]);
            }
        }

        $total = $links->count();
        $links = $links->slice(0, $limit);

        $viewParams = [
            'title' => $this->getTitle(),
            'links' => $links,
            'hasMore' => $total > $links->count()
        ];

        return $this->renderer('ld_widget_new_links', $viewParams);
    }

	public function verifyOptions(\XF\Http\Request $request, array &$options, &$error = null)
	{
		$options = $request->filter([
			'limit' => 'uint'
		]);
		if ($options['limit'] < 1)
		{
			$options['limit'] = 1;
		}

		return true;
	}
}