<?php

namespace xenMade\LD\XF\Pub\Controller;

class Watched extends XFCP_Watched
{
	public function actionLinkDirectory()
	{
		$this->setSectionContext('ld');

		$page = $this->filterPage();
		$perPage = $this->options()->ldLinksPerPage;

		/** @var \xenMade\LD\Repository\Link $linkRepo */
		$linkRepo = $this->repository('xenMade\LD:Link');
		$finder = $linkRepo->findLinksForWatchedList();

		$total = $finder->total();
        $links = $finder->limitByPage($page, $perPage)->fetch();

		$viewParams = [
			'page' => $page,
			'perPage' => $perPage,
			'total' => $total,
			'links' => $links->filterViewable()
		];

		return $this->view('xenMade\LD:Watched\Links', 'ld_watched_links', $viewParams);
	}

	public function actionLinkDirectoryManage()
	{
		$this->setSectionContext('ld');

		if (!$state = $this->filter('state', 'str'))
		{
			return $this->redirect($this->buildLink('watched/link-directory'));
		}

		if ($this->isPost())
		{
			/** @var \xenMade\LD\Repository\LinkWatch $linkWatchRepo */
            $linkWatchRepo = $this->repository('xenMade\LD:LinkWatch');

			if ($action = $this->getLinkWatchActionConfig($state, $updates))
			{
                $linkWatchRepo->setWatchStateForAll(\XF::visitor(), $action, $updates);
			}

			return $this->redirect($this->buildLink('watched/link-directory'));
		}
		else
		{
			$viewParams = [
				'state' => $state
			];
			return $this->view('xenMade\LD:Watched\LinkManage', 'watched_links_manage', $viewParams);
		}
	}

	public function actionLinkDirectoryUpdate()
	{
		$this->assertPostOnly();
		$this->setSectionContext('ld');

		/** @var \xenMade\LD\Repository\LinkWatch $watchRepo */
		$watchRepo = $this->repository('xenMade\LD:LinkWatch');

		$inputAction = $this->filter('watch_action', 'str');
		$action = $this->getLinkWatchActionConfig($inputAction, $config);

		if ($action)
		{
			$ids = $this->filter('ids', 'array-uint');
			$links = $this->em()->findByIds('xenMade\LD:Link', $ids);
			$visitor = \XF::visitor();

			/** @var \xenMade\LD\Entity\Link $link */
			foreach ($links AS $link)
			{
				$watchRepo->setWatchState($link, $visitor, $action, $config);
			}
		}

		return $this->redirect(
			$this->getDynamicRedirect($this->buildLink('watched/link-directory'))
		);
	}


	public function actionLinkDirectoryCategories()
	{
		$this->setSectionContext('ld');

		$watchedFinder = $this->finder('xenMade\LD:CategoryWatch');
		$watchedCategories = $watchedFinder->where('user_id', \XF::visitor()->user_id)
			->keyedBy('category_id')
			->fetch();

		/** @var \xenMade\LD\Repository\Category $categoryRepo */
		$categoryRepo = $this->repository('xenMade\LD:Category');
		$categories = $categoryRepo->getViewableCategories();
		$categoryTree = $categoryRepo->createCategoryTree($categories);
		$categoryExtras = $categoryRepo->getCategoryListExtras($categoryTree);

		$viewParams = [
			'watchedCategories' => $watchedCategories,

			'categoryTree' => $categoryTree,
			'categoryExtras' => $categoryExtras
		];
		return $this->view('xenMade\LD:Watched\Categories', 'ld_watched_categories', $viewParams);
	}

	public function actionLinkDirectoryCategoriesUpdate()
	{
		$this->assertPostOnly();
		$this->setSectionContext('ld');

		/** @var \xenMade\LD\Repository\CategoryWatch $watchRepo */
		$watchRepo = $this->repository('xenMade\LD:CategoryWatch');

		$inputAction = $this->filter('watch_action', 'str');
		$action = $this->getCategoryWatchActionConfig($inputAction, $config);

		if ($action)
		{
			$visitor = \XF::visitor();

			$ids = $this->filter('ids', 'array-uint');
			$categories = $this->em()->findByIds('xenMade\LD:Category', $ids);

			/** @var \xenMade\LD\Entity\Category $category */
			foreach ($categories AS $category)
			{
				$watchRepo->setWatchState($category, $visitor, $action, $config);
			}
		}

		return $this->redirect(
			$this->getDynamicRedirect($this->buildLink('watched/link-categories'))
		);
	}


	protected function getCategoryWatchActionConfig($inputAction, array &$config = null)
	{
		$config = [];

		$parts = explode(':', $inputAction, 2);

		$inputAction = $parts[0];
		$boolSwitch = isset($parts[1]) ? ($parts[1] == 'on') : false;

		switch ($inputAction)
		{
			case 'send_email':
			case 'send_alert':
			case 'include_children':
				$config = [$inputAction => $boolSwitch];
				return 'update';

			case 'delete':
				return 'delete';

			default:
				return null;
		}
	}

    protected function getLinkWatchActionConfig($inputAction, array &$config = null)
    {
        $config = [];

        switch ($inputAction)
        {
            case 'email_subscribe:on':
                $config = ['email_subscribe' => 1];
                return 'update';

            case 'email_subscribe:off':
                $config = ['email_subscribe' => 0];
                return 'update';

            case 'delete':
                return 'delete';

            default:
                return null;
        }
    }
}
if(false)
{
    class XFCP_Watched extends \XF\Pub\Controller\Watched {}
}